from datetime import datetime
import os.path
import pygame
from pygame.locals import *

class WalkTestAdvanced:
    def __init__(self, sett):
        self.directory = sett.directory
        self.time = sett.time
        self.direction = sett.direction
        self.bgtype = sett.background
        self.size = (sett.width, sett.height)

        self.frames = {}

    def file2surface(self, dirname, files):
        for i in range(len(files)):
            img = pygame.image.load(os.path.join(self.directory,
                                                 dirname, files[i]))
            files[i] = (img.convert_alpha(),
                        img.get_width(),
                        img.get_height())
        return files

    def path_walk(self, name, dirname, files):
        self.frames[name] = self.file2surface(dirname, sorted(files))

    def start(self):
        pygame.display.init()

        self.screen = pygame.display.set_mode(self.size)

        for x in 'lt', 'ct', 'rt', 'lm', 'rm', 'lb', 'cb', 'rb':
            os.path.walk(os.path.join(self.directory, x), self.path_walk, x)

        self.bgsurface = pygame.Surface(self.screen.get_size()).convert()

        self.clock = pygame.time.Clock()
        self.loop()

    def microseconds(self, tdelta):
        return tdelta.microseconds

    def loop(self):
        done = False
        i = 0
        cflow = self.bgtype == 'colorflow'
        if cflow:
            cr = 255
            cg = 0
            cb = 0
        else:
            color = self.bgtype

        time = datetime.now()
        while not done:
            self.clock.tick(30)
            if cflow:
                color = (cr, cg, cb)
            self.bgsurface.fill(color)
            self.screen.blit(self.bgsurface, (0, 0))

            if i >= len(self.frames[self.direction]):
                i = 0
            img = self.frames[self.direction][i]
            self.screen.blit(img[0], ((300 - img[1]) / 2,
                                      (600 - img[2]) / 2))

            pygame.display.flip()
            if cflow:
                if cr == 255 and cg < 255 and cb == 0:
                    cg += 5
                elif cr > 0 and cg == 255 and cb == 0:
                    cr -= 5
                elif cr == 0 and cg == 255 and cb < 255:
                    cb += 5
                elif cr == 0 and cg > 0 and cb == 255:
                    cg -= 5
                elif cr < 255 and cg == 0 and cb == 255:
                    cr += 5
                elif cr == 255 and cg == 0 and cb > 0:
                    cb -= 5
                if cr > 255: cr = 255
                elif cr < 0: cr = 0
                if cg > 255: cg = 255
                elif cg < 0: cg = 0
                if cb > 255: cb = 255
                elif cb < 0: cb = 0

            for event in pygame.event.get():
                if event.type == QUIT:
                    done = True
                elif event.type == KEYDOWN:
                    if event.key == K_ESCAPE:
                        done = True
                    elif event.key == K_KP7 or event.key == K_7:
                        self.direction = 'lt'
                    elif event.key == K_KP8 or event.key == K_8:
                        self.direction = 'ct'
                    elif event.key == K_KP9 or event.key == K_9:
                        self.direction = 'rt'
                    elif event.key == K_KP4 or event.key == K_4:
                        self.direction = 'lm'
                    elif event.key == K_KP6 or event.key == K_6:
                        self.direction = 'rm'
                    elif event.key == K_KP1 or event.key == K_1:
                        self.direction = 'lb'
                    elif event.key == K_KP2 or event.key == K_2:
                        self.direction = 'cb'
                    elif event.key == K_KP3 or event.key == K_3:
                        self.direction = 'rb'

            if self.microseconds(datetime.now() - time) > self.time:
                i = (i + 1) % len(self.frames[self.direction])
                time = datetime.now()



def goto(event):
    game = event.args[0]
    sys = game.sys
    sys.signalactions.clear('beforeworldrun')
    try:
        plcnum = int(sys.etc.arguments[0])
    except IndexError:
        plcnum = 0
    try:
        pos = eval('[' + sys.etc.arguments[1] + ']')
    except Exception:
        pos = None
    game.world.set_place(plcnum, pos)

def main():
    action('beforegamerun', goto)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Dililatum: a quest system for simple RPGs
# Copyright (C) 2010  Niels Serup

# This file is part of Dililatum.
#
# Dililatum is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Dililatum is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Dililatum.  If not, see <http://www.gnu.org/licenses/>.

##[ Name        ]## dialog
##[ Maintainer  ]## Niels Serup <ns@metanohi.org>
##[ Description ]## Contains the Dialog class whose purpose is to
                  # contain text and menus and to be drawn onto the
                  # screen

import pygame
from pygame.locals import *

class Dialog:
    def __init__(self, world, data=None):
        self.world = world

        self.surf = None
        if data is not None:
            self.use_data(data)

    def use_data(self, data):
        pass

    def draw(self, surf=None):
        if self.surf is None: return
        if surf is None:
            surf = self.world.screen
        surf.blit(self.surf, (0, 0))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Dililatum: a quest system for simple RPGs
# Copyright (C) 2010  Niels Serup

# This file is part of Dililatum.
#
# Dililatum is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Dililatum is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Dililatum.  If not, see <http://www.gnu.org/licenses/>.

##[ Name        ]## world
##[ Maintainer  ]## Niels Serup <ns@metanohi.org>
##[ Description ]## Contains the World class, controlling display
                  # output as well as key detection

from datetime import datetime
import os.path
import pygame
from pygame.locals import *
from dililatum.character import Character, EmptyCharacter
from dililatum.object import Object
from dililatum.place import Place
from dililatum.font import Font
from dililatum.sound import Sound
from dililatum.msgbox import MessageBox, MessageContainer
from dililatum.statusprinter import StatusPrinter
from dililatum.various import thread

microseconds = lambda tdelta: tdelta.microseconds
class TimeCounter:
    def __init__(self, obj, func):
        self.obj = obj
        self.func = func
        self.start = self.reset

    def reset(self):
        self.time = datetime.now()

    def think(self):
        if microseconds(datetime.now() - self.time) > \
                self.obj.duration:
            self.func()
            self.reset()

class Duration:
    def __init__(self, dur):
        self.duration = dur * 1000

class World:
    default_character_moving_keys = dict(
        cb=[K_DOWN],
        ct=[K_UP],
        lm=[K_LEFT],
        rm=[K_RIGHT],
        lb=[K_LEFT, K_DOWN],
        rt=[K_RIGHT, K_UP],
        lt=[K_LEFT, K_UP],
        rb=[K_RIGHT, K_DOWN]
    )

    def __init__(self, stm, size, **oargs):
        def get(key, default):
            try: return oargs[key]
            except KeyError: return default
        self.sys = stm
        self.size = size
        self.real_size = tuple(self.size[:])
        self.loaded_images = {}
        self.objects = []
        self.sounds = []
        self.characters = []
        self.leading_character = EmptyCharacter()
        self.leading_character_direction = None
        self.character_moving_keys = get('charkeys',
                                         self.default_character_moving_keys)
        self.walking_speed = get('walkspeed', Duration(100))
        self.msgboxes = []
        self.default_msgbox = None
        self.counters = []
        self.places = []
        self.current_place = None
        self.running = False
        self.quitting = False
        self.status = StatusPrinter('WORLD', self.sys.etc, 'cyan', 'blue')
        self.link_event = self.sys.gameactions.add
        self.unlink_event = self.sys.gameactions.remove
        self.keys_down = []
        self.screen_offset = [0, 0]
        self.screen_bars = [None, None]
        self.keys_locked = False
        self.event = {}
        self.sys.emit_signal('afterworldinit', self)

    def start(self):
        self.sys.emit_signal('beforeworldstart', self)
        self.status('Starting up...')

        self.link_event(QUIT, self.quit)
        self.link_event(KEYDOWN, self.key_builtin)
        self.link_event(KEYDOWN, self.lead_walk)
        self.link_event(KEYUP, self.lead_walk)

        self.counters.append(TimeCounter(self.walking_speed, self.check_lead_walk))

        self.pygame_init()
        self.create_screen()
        self.fill_background((0, 0, 0))
        self.set_caption(self.sys.game.name)

        self.innerclock = pygame.time.Clock()
        self.sys.emit_signal('afterworldstart', self)

    def pygame_init(self):
        pygame.display.init()
        pygame.font.init()
        pygame.mixer.pre_init(44100) # Sound files must be resampled
                                     # to 44.1 kHz
        pygame.mixer.init()

    def load_image(self, path, alpha=False):
        p = os.path.abspath(path)
        if p not in self.loaded_images.values():
            img = pygame.image.load(p)
            if alpha:
                img = img.convert_alpha()
            else:
                img = img.convert()
            self.loaded_images[p] = img
        else:
            img = self.loaded_images[p]
        return img

    def create_screen(self):
        self.screen_bars = [None, None]
        etc = self.sys.etc
        if etc.fullscreen:
            flags = FULLSCREEN
            if etc.hwaccel:
                flags = flags | HWSURFACE
            if etc.doublebuf:
                flags = flags | DOUBLEBUF
            self.screen = pygame.display.set_mode(self.size, flags)
        elif etc.fakefullscreen or etc.size is not None:
            barsize = None
            if etc.fakefullscreen or (etc.size is not None and etc.size[0] is None and etc.size[1] is None):
                try:
                    info = pygame.display.Info()
                    size = info.current_w, info.current_h
                    if size[0] == -1: # in this case, size[1] will also be -1
                        self.sys.error('your SDL is too old for width and height detection', False)
                except Exception:
                    self.sys.error('your PyGame is too old for width and height detection', False)
            else:
                size = list(etc.size)
            if size[0] is None:
                self.sys.etc.zoom = size[1] / float(self.size[1])
                size[0] = int(self.size[0] * etc.zoom)
            elif size[1] is None:
                self.sys.etc.zoom = size[0] / float(self.size[0])
                size[1] = int(self.size[1] * etc.zoom)
            else:
                scales = [size[i] / float(self.size[i]) for i in range(2)]
                if scales[0] < scales[1]: a = 0; b = 1
                else:                     a = 1; b = 0

                self.sys.etc.zoom = scales[a]
                self.screen_offset[b] = int((size[b] - self.size[b] * etc.zoom) / 2)
                barsize = [0, 0]
                barsize[a] = size[a]
                barsize[b] = self.screen_offset[b]
            self.real_size = size
            self.status('Modified size of game is: %dx%d' % tuple(self.real_size))
            flags = None
            if not etc.border or etc.fakefullscreen:
                flags = NOFRAME
            if etc.doublebuf:
                if flags is not None:
                    flags = flags | DOUBLEBUF
                else:
                    flags = DOUBLEBUF
            try:
                self.screen = pygame.display.set_mode(self.real_size, flags)
            except pygame.error:
                if not etc.border or etc.fakefullscreen:
                    flags = NOFRAME
                else:
                    flags = 0
                self.screen = pygame.display.set_mode(self.real_size, flags)
            if barsize is not None:
                self.screen_bars[b] = pygame.Surface(barsize).convert()
        else:
            if etc.zoom != 1:
                self.real_size = [int(x * etc.zoom) for x in self.size]
                self.status('Scaled size of game is: %dx%d' % tuple(self.real_size))
            flags = None
            if not etc.border or etc.fakefullscreen:
                flags = NOFRAME
            if etc.doublebuf:
                if flags is not None:
                    flags = flags | DOUBLEBUF
                else:
                    flags = DOUBLEBUF
            try:
                self.screen = pygame.display.set_mode(self.real_size, flags)
            except pygame.error:
                if not etc.border or etc.fakefullscreen:
                    flags = NOFRAME
                else:
                    flags = 0
                self.screen = pygame.display.set_mode(self.real_size, flags)

        self.bgsurface = pygame.Surface(self.real_size).convert()

    def fill_background(self, color):
        self.bgsurface.fill(color)
        for x in self.screen_bars:
            if x is not None:
                x.fill(color)

    def set_caption(self, caption):
        pygame.display.set_caption(caption)

    def set_icon(self, surf):
        pygame.display.set_icon(surf)

    def load_icon(self, path):
        self.set_icon(self.load_image(path, True))

    def blit(self, surf, pos=(0, 0), area=None):
        if surf is None: return
        surf = pygame.transform.smoothscale(
            surf, [int(x * self.sys.etc.zoom)
                   for x in surf.get_size()])
        if area is not None:
            area.width *= self.sys.etc.zoom
            area.height *= self.sys.etc.zoom
        self.screen.blit(surf, [self.screen_offset[i] + pos[i] *
                                self.sys.etc.zoom for i in range(2)], area)

    def flip(self):
        pygame.display.flip()

    def lock_keys(self):
        self.keys_locked = True

    def unlock_keys(self):
        self.keys_locked = False

    def create_font(self, **oargs):
        return Font(self, **oargs)

    def create_character(self, idname, datafiles, **oargs):
        self.sys.emit_signal('beforecharactercreate', self, idname)
        char = Character(self, idname, datafiles, **oargs)
        self.sys.emit_signal('aftercharactercreate', char)
        return char

    def add_character(self, char):
        self.sys.emit_signal('beforecharacteradd', char)
        self.characters.append(char)
        counter = TimeCounter(char, char.next_step)
        self.counters.append(counter)
        if self.running:
            counter.start()
        self.sys.emit_signal('aftercharacteradd', char)

    def create_msgbox(self, **oargs):
        self.sys.emit_signal('beforemsgboxcreate', self, oargs)
        msgbox = MessageBox(self, **oargs)
        self.sys.emit_signal('aftermsgboxcreate', msgbox)
        return msgbox

    def add_msgbox(self, msgbox):
        self.sys.emit_signal('beforemsgboxadd', msgbox)
        self.msgboxes.append(msgbox)
        self.sys.emit_signal('aftermsgboxadd', msgbox)

    def show_message(self, msg, **oargs):
        msgbox = oargs.get('msgbox') or self.default_msgbox
        MessageContainer(msgbox).show_message(msg, **oargs)

    def show_question(self, *args, **oargs):
        msgbox = oargs.get('msgbox') or self.default_msgbox
        oargs['pointer'] = msgbox.poimg
        oargs['posize'] = msgbox.posize
        MessageContainer(msgbox).show_question(*args, **oargs)

    def set_leading_character(self, char):
        self.leading_character = char

    def check_lead_walk(self):
        if self.leading_character_direction is None:
            self.leading_character.stop()
        else:
            self.leading_character.walk(self.leading_character_direction)

    def lead_walk(self, event):
        if event.type == KEYDOWN:
            self.keys_down.append(event.key)
        elif event.type == KEYUP:
            try:
                self.keys_down.remove(event.key)
            except ValueError:
                pass

        if not self.keys_locked:
            self.leading_character_direction = \
                self.get_value_of_key_dict(self.character_moving_keys)
        else:
            self.leading_character_direction = None

    def get_value_of_key_dict(self, keydict):
        result = None
        result_key_numbers = None
        result_points = None
        for value, keys in keydict.items():
            p = []
            tp = 0
            tn = 0
            ok = True
            for k in keys:
                i = 0
                ok = False
                for d in self.keys_down:
                    if k == d:
                        tp += i
                        tn += 1
                        ok = True
                        break
                    i += 1
                if not ok:
                    break
            if ok:
                if result_key_numbers is not None:
                    tngo = tn > result_key_numbers
                    tn2go = tn == result_key_numbers
                else:
                    tngo = True
                if not tngo:
                    if result_points is not None:
                        tpgo = tp < result_points
                    else:
                        tpgo = True

                if tngo or (tn2go and tpgo):
                    result = value
                    result_points = tp
                    result_key_numbers = tn

        return result

    def remove_character(self, char):
        if 'id' not in dir(char):
            char = None
            for t in self.characters:
                if t.id == char:
                    char = t
            if char is None:
                return False

        self.sys.emit_signal('beforecharacterremove', char)
        self.characters.remove(char)
        self.remove_counter(char)
        self.sys.emit_signal('aftercharacterremove', char)
        return True

    def remove_counter(self, obj):
        try:
            self.counters.remove(obj)
            return True
        except Exception:
            for t in self.counters:
                if t.obj == obj:
                    self.counters.remove(t)
                    return True
            return False

    def create_sound(self, path):
        self.sys.emit_signal('beforesoundcreate', self, path)
        snd = Sound(self, path)
        self.sys.emit_signal('aftersoundcreate', self, snd)
        return snd

    def add_sound(self, snd):
        self.sys.emit_signal('beforesoundadd', self, snd)
        self.sounds.append(snd)
        self.sys.emit_signal('aftersoundadd', self, snd)

    def create_object(self, *args, **oargs):
        self.sys.emit_signal('beforeobjectcreate', self)
        args = list(args)
        args.insert(0, self)
        obj = Object(*args, **oargs)
        self.sys.emit_signal('afterobjectcreate', obj)
        return obj

    def create_place(self, *args, **oargs):
        self.sys.emit_signal('beforeplacecreate', self)
        args = list(args)
        args.insert(0, self)
        place = Place(*args, **oargs)
        self.sys.emit_signal('afterplacecreate', place)
        return place

    def add_place(self, place):
        self.sys.emit_signal('beforeplaceadd', place)
        self.places.append(place)
        self.sys.emit_signal('afterplaceadd', place)

    def set_place(self, place, npos=None, direction=None):
        try:
            curbgsnds = self.current_place.bgsounds
        except Exception:
            curbgsnds = []
        if 'draw' in dir(place):
            self.current_place = place
        else:
            try:
                self.current_place = self.places[place]
            except IndexError:
                pass
        if self.leading_character is None:
            return

        if npos is None:
            npos = self.leading_character.default_position
        npos = npos[:]
        if npos[0] < 0:
            npos[0] = int(
                self.size[0] + npos[0] -
                place.char_size(npos) *
                self.leading_character.maxsize[0]
            )
        self.leading_character.stop()

        if direction is not None:
            if direction[0] == 'l':
                npos[0] = int(npos[0] - (place.char_size(npos) / 2) *
                              self.leading_character.maxsize[0])
            elif direction[0] == 'r':
                npos[0] = int(npos[0] + (place.char_size(npos) / 2) *
                              self.leading_character.maxsize[0])
            self.leading_character.original_direction = direction
            self.leading_character.direction = direction
            self.leading_character_direction = direction
        self.leading_character.original_position = npos
        self.leading_character.position = npos[:]
        self.leading_character.modified_position = npos[:]

        newbgsnds = self.current_place.bgsounds
        for x in curbgsnds:
            if x not in newbgsnds:
                x.stop()
        for x in newbgsnds:
            if not x.is_playing:
                thread(x.play)

    def get_center(self):
        return [x / 2 for x in self.size]

    def set_default_msgbox(self, msgbox):
        self.default_msgbox = msgbox

    def draw(self):
        self.screen.blit(self.bgsurface, (0, 0))
        if self.current_place is None:
            return
        self.current_place.draw()

        objs = self.characters[:]
        objs.extend(self.current_place.objects)
        objs.extend(self.objects)
        objs.sort(key=lambda o: o.get_bottom_area()[1])
        for x in objs:
            if x.visible:
                x.draw()

        for x in self.msgboxes:
            x.draw()

        if self.screen_bars[0] is not None:
            self.screen.blit(self.screen_bars[0], (0, 0))
            self.screen.blit(self.screen_bars[0],
                             (self.real_size[0] -
                              self.screen_bars[0].get_size()[0], 0))
        if self.screen_bars[1] is not None:
            self.screen.blit(self.screen_bars[1], (0, 0))
            self.screen.blit(self.screen_bars[1],
                             (0, self.real_size[1] -
                              self.screen_bars[1].get_size()[1]))

        pygame.display.flip()

    def run(self):
        self.sys.emit_signal('beforeworldrun', self)
        self.status('Running...')
        self.running = True

        for c in self.counters:
            c.start()

        while not self.quitting:
            self.innerclock.tick(30)
            self.sys.emit_signal('beforegameloop', self)

            for event in pygame.event.get():
                self.event[event.type] = event
                self.sys.emit_event(event.type, event)

            for c in self.counters:
                c.think()

            self.draw()
            self.sys.emit_signal('aftergameloop', self)

        self.sys.emit_signal('afterworldrun', self)

    def quit(self, event):
        self.quitting = True

    def key_builtin(self, event):
        if event.key == K_SCROLLOCK:
            self.leading_character.reset_position()
        if event.key == K_ESCAPE:
            self.quit(event)

    def end(self):
        self.sys.emit_signal('beforeworldend', self)
        self.status('Stopping...')
        self.sys.emit_signal('afterworldend', self)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Dililatum: a quest system for simple RPGs
# Copyright (C) 2010  Niels Serup

# This file is part of Dililatum.
#
# Dililatum is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Dililatum is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Dililatum.  If not, see <http://www.gnu.org/licenses/>.

##[ Name        ]## msgbox
##[ Maintainer  ]## Niels Serup <ns@metanohi.org>
##[ Description ]## Makes it possible to display messages and speech
                  # throughout the game

import pygame
from pygame.locals import *
import os.path

HEADSIZE = 75

class MessageBox:
    def __init__(self, world, **oargs):
        def get(key, default):
            try: return oargs[key]
            except KeyError: return default

        self.world = world
        self.path = get('path', None)
        self.font = get('font', None)
        self.default_pos = get('pos', None)
        self.headtextrel = get('headtextrel', 0)
        # 0: | HEAD | TEXT... |
        # 1: | TEXT... | HEAD |
        # 2: |   HEAD  |
        #    | TEXT... |
        # 3: | TEXT... |
        #    |   HEAD  |
        self.size = get('size', (620, 120))
        self.posize = get('posize', (10, 10))
        self.visible = False

        self.msgcontainers = []

        if self.path is not None:
            try:
                self.bgimg = \
                    self.world.load_image(os.path.normpath(
                        os.path.join(self.path, 'bgimg.png')), True)
                self.size = self.bgimg.get_size()
            except Exception:
                self.bgimg = None
            try:
                self.poimg = \
                    self.world.load_image(os.path.normpath(
                        os.path.join(self.path, 'pointer.png')), True)
                self.posize = self.poimg.get_size()
            except Exception:
                self.poimg = None

            if self.default_pos is None:
                self.default_pos = ((self.world.size[0] - self.size[0]) / 2, 10)
            if self.headtextrel == 0:
                self.headpos = (10, (self.size[1] - HEADSIZE) / 2)
                self.textpos = (20 + HEADSIZE, 10)
                self.textsize = (self.size[0] - 35 - HEADSIZE,
                                 self.size[1] - 20)
            elif self.headtextrel == 1:
                self.textpos = (10, 10)
                self.textsize = (self.size[0] - 35 - HEADSIZE,
                                 self.size[1] - 20)
                self.headpos = (self.textpos[0] + self.textsize[0] +
                                10, (self.size[1] - HEADSIZE) / 2)
            elif self.headtextrel == 2:
                self.headpos = ((self.size[0] - HEADSIZE) / 2, 10)
                self.textpos = (10, 20 + HEADSIZE)
                self.textsize = (self.size[0] - 20,
                                 self.size[1] - 35 - HEADSIZE)
            elif self.headtextrel == 3:
                self.textpos = (10, 10)
                self.textsize = (self.size[0] - 20,
                                 self.size[1] - 35 - HEADSIZE)
                self.headpos = ((self.size[0] - HEADSIZE) / 2,
                                self.textpos[1] + self.textsize[1] + 10)
        else:
            self.bgimg = None
            self.poimg = None
            if self.default_pos is None:
                self.default_pos = (10, 10)
            self.headpos = (20, 20)
            self.textpos = (20 + HEADSIZE, 20)

    def draw(self):
        for x in self.msgcontainers:
            x.draw()

class MessageContainer:
    def __init__(self, msgbox, pos=None):
        self.msgbox = msgbox
        self.world = self.msgbox.world
        self.head = None
        self.text = None

        self.visible = False

    # Messages
    def show_message(self, text=None, **oargs):
        self.head = oargs.get('head') or None
        self.pos = oargs.get('pos') or self.msgbox.default_pos
        self.temp_endaction = oargs.get('endaction') or None
        if text is not None:
            self.text = text
            self.textsurfs = self.msgbox.font.write(
                text, split=self.msgbox.textsize)
            self.surfnum = 0
            self.world.lock_keys()
            self.world.link_event(KEYDOWN,
                                  self.show_more_message_text)
        if self.head or self.text:
            self.msgbox.msgcontainers.append(self)
            self.visible = True

    def show_more_message_text(self, event):
        if event.key in (K_RETURN, K_SPACE, K_KP_ENTER):
            self.surfnum += 1
        if self.surfnum == len(self.textsurfs):
            self.hide()
            self.world.unlink_event(KEYDOWN,
                                    self.show_more_message_text)
            self.world.unlock_keys()
            if self.temp_endaction is not None:
                self.temp_endaction[0](*self.temp_endaction[1:])
            del self.temp_endaction

    # Questions with answers
    def show_question(self, question=None, *answers, **oargs):
        self.head = oargs.get('head') or None
        posize = oargs.get('posize') or None
        self.pointer = oargs.get('pointer') or None
        self.pos = oargs.get('pos') or self.msgbox.default_pos
        self.temp_endaction = oargs.get('endaction') or None
        self.answers = answers
        if question is not None:
            self.text = question
            self.textsurfs, self.pposs, self.surf_of_ans = self.msgbox.font.write(
                self.text, split=self.msgbox.textsize,
                indent_size=posize, indent_text=[a[0] for a in self.answers])
            self.surfnum = 0
            self.ansnum = 0
            self.anslen = len(self.pposs)
            self.surfnumstart, self.surfnumend = \
                self.surf_of_ans[self.ansnum]
            self.surfnum = self.surfnumstart
            self.world.lock_keys()
            self.world.link_event(KEYDOWN,
                                  self.show_more_question_text)
        if self.head or self.text:
            self.msgbox.msgcontainers.append(self)
            self.visible = True

    def show_more_question_text(self, event):
        if event.key in (K_DOWN,):
            if self.surfnum < self.surfnumend:
                self.surfnum += 1
            elif self.ansnum < self.anslen - 1:
                self.ansnum += 1
                self.surfnumstart, self.surfnumend = \
                    self.surf_of_ans[self.ansnum]
                self.surfnum = self.surfnumstart
        elif event.key in (K_UP,):
            if self.surfnum > self.surfnumstart:
                self.surfnum -= 1
            elif self.ansnum > 0:
                self.ansnum -= 1
                self.surfnumstart, self.surfnumend = \
                    self.surf_of_ans[self.ansnum]
                self.surfnum = self.surfnumend

        if event.key in (K_RETURN, K_SPACE, K_KP_ENTER):
            self.hide()
            self.world.unlink_event(KEYDOWN,
                                    self.show_more_question_text)
            self.world.unlock_keys()
            if self.temp_endaction is not None:
                self.temp_endaction[0](*self.temp_endaction[1:])
            del self.temp_endaction
            a = self.answers[self.ansnum]
            a[1](*a[2:])

    def hide(self):
        self.msgbox.msgcontainers.remove(self)
        self.visible = False

    def draw(self):
        if not self.visible: return

        if self.msgbox.bgimg is not None:
            self.world.blit(self.msgbox.bgimg, self.pos)

        if self.head is not None:
            self.world.blit(self.head,
                            [self.msgbox.headpos[i] +
                             self.pos[i]
                             for i in range(2)])

        if self.text is not None:
            surf = self.textsurfs[self.surfnum]
            text_x = self.msgbox.textpos[0] + self.pos[0]
            text_y = self.pos[1] + self.msgbox.textpos[1] + \
                (self.msgbox.size[1] - surf.get_size()[1]) / 2
            pos = (text_x, text_y)
            self.world.blit(surf, pos)

            try:
                if self.surfnum == self.surfnumstart:
                    self.world.blit(
                        self.pointer, (text_x, text_y +
                                       self.pposs[self.ansnum][1]))
            except AttributeError:
                pass




#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Dililatum: a quest system for simple RPGs
# Copyright (C) 2010  Niels Serup

# This file is part of Dililatum.
#
# Dililatum is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Dililatum is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Dililatum.  If not, see <http://www.gnu.org/licenses/>.

##[ Name        ]## various
##[ Maintainer  ]## Niels Serup <ns@metanohi.org>
##[ Description ]## Various minor (but still important) parts


import sys
from threading import Thread

def read_file(filename, throw_exceptions=False):
    if not throw_exceptions:
        try:
            f = open(filename, 'U')
            return f.read()
        except Exception:
            return ''
    else:
        f = open(filename)
        return f.read()

def error(msg, cont=True, pre=None):
    errstr = msg + '\n'
    if pre is not None:
        errstr = pre + ': ' + errstr
    sys.stderr.write(errstr)
    if cont is not True:
        try:
            sys.exit(cont)
        except Exception:
            sys.exit(1)

def usable_error(msg, cont=True):
    error(msg, cont, 'dililatum: ')


class thread(Thread):
    def __init__(self, func, *args):
        Thread.__init__(self)
        self.func = func
        self.args = args
        self.setDaemon(True) # self.daemon = True
        self.start()

    def run(self):
        self.func(*self.args)


# Not running in daemon mode
class nthread(Thread):
    def __init__(self, func, *args):
        Thread.__init__(self)
        self.func = func
        self.args = args
        self.start()

    def run(self):
        self.func(*self.args)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Dililatum: a quest system for simple RPGs
# Copyright (C) 2010  Niels Serup

# This file is part of Dililatum.
#
# Dililatum is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Dililatum is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Dililatum.  If not, see <http://www.gnu.org/licenses/>.

##[ Name        ]## character
##[ Maintainer  ]## Niels Serup <ns@metanohi.org>
##[ Description ]## Contains the Character class, controlling how
                  # people and monsters alike move

from datetime import datetime
import os.path
import pygame
from pygame.locals import *

class Frame:
    def __init__(self, img):
        self.surf = img
        self.width = img.get_width()
        self.height = img.get_height()

class EmptyCharacter:
    def stop(self):
        pass

    def walk(self, direction):
        pass

class Character:
    def __init__(self, world, idname, datafiles, **oargs):
        def get(key, default):
            try: return oargs[key]
            except KeyError: return default
        self.id = idname
        self.name = get('name', None)
        self.world = world
        self.data = datafiles
        self.frames = {}
        self.walking = False
        self.step = 0
        self.original_direction = get('direction', 'cb')
        self.direction = self.original_direction[:]
        self.position = get('position', world.get_center())
        self.duration = get('duration', 200) * 1000
        self.movement = get('movement', dict(
                cb=(0, 3),
                ct=(0, -3),
                lm=(-3, 0),
                rm=(3, 0),
                lb=(-2.12, 2.12),
                rt=(2.12, -2.12),
                lt=(-2.12, -2.12),
                rb=(2.12, 2.12)
        ))
        self.use_shadow = get('shadow', True)
        self.shadow_details = get('shadowdetails', {})
        self.default_direction = 'cb'
        self.default_position = (
            self.world.size[0] / 2,
            int(self.world.size[1] * 0.9))
        self.original_position = get('position', self.default_position[:])
        self.position = self.position[:]
        self.modified_position = self.position[:]
        self.visible = get('visible', False)

        self.create()
        if self.use_shadow:
            self.create_shadow()

    def get_bottom_area(self):
        return self.modified_position[0] + self.get_size(self.position)[0] / 2, self.modified_position[1]

    def convert_files_to_surfaces(self, *files):
        frames = []
        for f in files:
            img = self.world.load_image(f, True)
            frames.append(Frame(img))
        return frames

    def create(self):
        for x in 'lt', 'ct', 'rt', 'lm', 'rm', 'lb', 'cb', 'rb':
            if x in self.data[1]: # Files and directories
                files = self.data[1][x][1] # 1 = files
                files = [os.path.join(self.data[0], x, t) for t in files]
                self.frames[x] = \
                    self.convert_files_to_surfaces(*files)

        if len(self.frames) < 8:
            self.fill_out_remaining_directions()

        maxw = 0
        maxh = 0
        for x in self.frames.values():
            for y in x:
                if y.width > maxw:
                    maxw = y.width
                if y.height > maxh:
                    maxh = y.height
        self.maxsize = (maxw, maxh)

        try:
            self.head = \
                self.world.load_image(os.path.join(self.data[0],
                                                   'head.png'), True)
        except Exception:
            self.head = None

    def fill_out_remaining_directions(self):
        tries = {
            'lt': ('ct', 'lm', 'rt', 'lb', 'rm', 'cb', 'rb'),
            'rt': ('ct', 'rm', 'lt', 'rb', 'lm', 'cb', 'lb'),
            'ct': ('lt', 'rt', 'lm', 'rm', 'lb', 'rb', 'cb'),
            'cb': ('lb', 'rb', 'lm', 'rm', 'lt', 'rt', 'ct'),
            'lm': ('lt', 'lb', 'ct', 'cb', 'rt', 'rb', 'rm'),
            'rm': ('rt', 'rb', 'ct', 'cb', 'lt', 'lb', 'lm'),
            'lb': ('cb', 'lm', 'rb', 'lt', 'rm', 'ct', 'rt'),
            'rb': ('cb', 'rm', 'lb', 'rt', 'lm', 'ct', 'lt')
            }
        for x in 'lt', 'ct', 'rt', 'lm', 'rm', 'lb', 'cb', 'rb':
            if x not in self.frames:
                ok = False
                i = 0
                while not ok:
                    try:
                        self.frames[x] = self.frames[tries[x][i]]
                        ok = True
                        break
                    except Exception:
                        if i == 7:
                            break
                        i += 1
                if not ok:
                    self.frames[x] = None

    def create_shadow(self):
        charsurf = self.frames['cb'][0]
        size = (charsurf.width - charsurf.width / 15, charsurf.height / 15)
        ssize = [s * 3 for s in size]
        rect = pygame.Rect((0, 0), ssize)
        surf = pygame.Surface(ssize).convert_alpha()
        surf.fill(pygame.Color(0, 0, 0, 0))
        pygame.draw.ellipse(surf, pygame.Color(0, 0, 0, 200), rect)
        surf = pygame.transform.smoothscale(surf, size)

        self.shadow = surf
        self.shadow_size = surf.get_size()

    def get_size(self, pos, img=None):
        resize = self.world.current_place.char_size(pos)
        if img is None:
            img = self.get_frame()
        w = int(img.width * resize)
        h = int(img.height * resize)
        return w, h, resize

    def get_frame(self):
        if self.walking:
            return self.frames[self.direction][self.step]
        else:
            return self.frames[self.direction][0]

    def next_step(self):
        self.step = (self.step + 1) % len(self.frames[self.direction])

    def stop(self):
        self.walking = False

    def is_reverse(self, direction):
        a = self.direction
        b = direction
        return (a == 'cb' and b in ('ct', 'rt', 'lt')) \
            or (a == 'ct' and b in ('cb', 'rb', 'lb')) \
            or (a == 'lm' and b in ('rm', 'rt', 'rb')) \
            or (a == 'rm' and b in ('lm', 'lt', 'lb')) \
            or (a == 'lt' and b == 'rb') \
            or (a == 'rt' and b == 'lb') \
            or (a == 'lb' and b == 'rt') \
            or (a == 'rb' and b == 'lt')

    def walk(self, direction):
        w, h, resize = self.get_size(self.position)
        size = w, h
        maxsize = [s * resize for s in self.maxsize]
        mov = self.movement[direction]
        pos = self.position
        pos_ok = False
        scale = 1.0

        while not pos_ok:
            npos = [int(pos[i] + mov[i] * resize * scale *
                        self.world.size[i] / 100.0) for i in range(2)]
            mpos = self.world.current_place.char_pos(npos)
            pos_ok = self.world.current_place.pos_ok(mpos, maxsize)

            scale -= .1
            if scale < 0.4:
                break
        if pos_ok:
            conti = True
            for o in self.world.current_place.objects:
                t = o.check_if_action_needed(npos, maxsize)
                if not t: conti = False
            if conti:
                self.direction = direction
                self.position = npos
                self.modified_position = mpos
                self.walking = True
        else:
            self.stop()

    def set_position(self, pos):
        self.position = pos
        self.modified_position = self.world.current_place.char_pos(pos)

    def reset_position(self):
        self.position = self.original_position[:]
        self.modified_position = self.original_position[:]
        self.direction = self.original_direction
        w, h, resize = self.get_size(self.modified_position)
        for o in self.world.current_place.objects:
            o.check_if_action_needed(self.position, (w, h))

    def say(self, msg, **oargs):
        if self.name is not None:
            msg = self.name + ': ' + msg
        oargs = oargs.copy()
        oargs['head'] = self.head
        self.world.show_message(msg, **oargs)

    def ask(self, question, *answers, **oargs):
        if self.name is not None:
            question = self.name + ': ' + question
        oargs = oargs.copy()
        oargs['head'] = self.head
        self.world.show_question(question, *answers, **oargs)

    def hide(self):
        self.visible = False

    def show(self):
        self.visible = True

    def draw(self, surf=None):
        if not self.visible: return False
        img = self.get_frame()
        pos = self.modified_position

        w, h, r = self.get_size(pos, img)
        img = pygame.transform.smoothscale(img.surf, (w, h))
        if surf is None:
            surf = self.world
        if self.use_shadow:
            ssize = [int(s * r) for s in self.shadow_size]
            shad = pygame.transform.smoothscale(
                self.shadow, ssize)
            shad_pos = (pos[0] - w / 2 + (w - ssize[0]) / 2, pos[1] - h / 15)
            surf.blit(shad, shad_pos)
        surf.blit(img, (pos[0] - w / 2, pos[1] - h))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Dililatum: a quest system for simple RPGs
# Copyright (C) 2010  Niels Serup

# This file is part of Dililatum.
#
# Dililatum is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Dililatum is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Dililatum.  If not, see <http://www.gnu.org/licenses/>.

##[ Name        ]## place
##[ Maintainer  ]## Niels Serup <ns@metanohi.org>
##[ Description ]## Contains the generic form of the Place class

import pygame
from pygame.locals import *
from dililatum.bitmap import BitMap

class Place:
    def __init__(self, world, imgfile=None, posfile=None, **oargs):
        def get(key, default):
            try: return oargs[key]
            except KeyError: return default
        self.world = world
        self.imgfile = imgfile
        self.posfile = posfile
        self.name = None
        self.objects = []
        self.objects = get('objs', [])
        self.obj_names = {}
        self.dir_objects = {}
        self.bgsounds = get('bgsnd', [])

        if self.world.sys.etc.loadwait:
            self.load_imgfile()
            self.load_posfile()
        else:
            self.surf = None
            self.posoks = None

    def load_imgfile(self):
        if self.imgfile is not None:
            try:
                self.surf = pygame.image.load(self.imgfile).convert()
            except Exception:
                pass

    def load_posfile(self):
        if self.posfile is not None:
            try:
                bm = BitMap(*self.world.size)
                bm.load(self.posfile)
                self.posoks = bm
            except Exception:
                pass

    def add_object(self, obj):
        self.objects.append(obj)

    def set_direction_object(self, direction, obj):
        self.dir_objects[direction] = obj

    def remove_object(self, obj):
        self.objects.remove(obj)

    def char_size(self, pos):
        return 1.0

    def char_pos(self, pos):
        return pos

    def pos_ok(self, pos, size):
        if self.posoks is None:
            self.load_posfile()
        if self.posoks is None: # Still..
            return True
        else:
            height = self.char_size(pos) * \
                self.world.leading_character.get_frame().height / 20
            s0 = size[0] / 2
            posmh = pos[1] - height
            return self.posoks.get(*pos) and \
                self.posoks.get(pos[0] - s0, pos[1]) and \
                self.posoks.get(pos[0] + s0, pos[1]) and \
                self.posoks.get(pos[0], posmh) and \
                self.posoks.get(pos[0] - s0, posmh) and \
                self.posoks.get(pos[0] + s0, posmh) and \
                self.posoks.get(pos[0], (pos[1] + posmh) / 2)

    def draw(self, surf=None):
        if surf is None:
            surf = self.world
        if self.surf is None:
            self.load_imgfile()
        surf.blit(self.surf, (0, 0))




#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Dililatum: a quest system for simple RPGs
# Copyright (C) 2010  Niels Serup

# This file is part of Dililatum.
#
# Dililatum is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Dililatum is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Dililatum.  If not, see <http://www.gnu.org/licenses/>.

##[ Name        ]## font
##[ Maintainer  ]## Niels Serup <ns@metanohi.org>
##[ Description ]## Contains the Font class, which makes it possible
                  # to write text

import pygame
from pygame.locals import *
import math

class Font:
    def __init__(self, world, **oargs):
        def get(key, default):
            try: return oargs[key]
            except KeyError: return default

        self.world = world
        self.path = get('path', None)
        self.size = get('size', 30)

        self.create_font()

    def create_font(self):
        self.font = pygame.font.Font(self.path, self.size)

    def split_text(self, text, width):
        spl = text.split('\n')
        r = []
        for x in spl:
            r.extend(self.split_text_pieces(x, width))
        return r

    def split_text_pieces(self, text, width):
        t = text[:]
        ts = []
        while t:
            tt = t[:]
            s = False
            while self.font.size(tt)[0] > width:
                nf = tt.rfind(' ')
                if nf != -1:
                    tt = tt[:nf]
                    s = True
                else:
                    tt = tt[:-1]
                    s = False
            ts.append(tt)
            if s:
                t = t[len(tt) + 1:]
            else:
                t = t[len(tt):]
        return ts

    def write(self, text, **oargs):
        color = oargs.get('color') or (0, 0, 0)
        antialias = oargs.get('antialias') or True
        split = oargs.get('split') or False
        indent_size = oargs.get('indent_size') or None
        indent_text = oargs.get('indent_text') or None
        if indent_text is not None and indent_size is None:
            indent_size = (20, 20)

        if not split:
            return self.font.render(text, antialias, color)
        else:
            fh = self.font.size('')[1]
            ts = self.split_text(text, split[0])
            tslen = len(ts)
            if indent_text:
                tsi = []
                tsisimp = []
                i = 0
                j = 0
                for x in indent_text:
                    st = self.split_text(x, split[0] -
                               indent_size[0])
                    tsi.append(st)
                    k = 0
                    for y in st:
                        tsisimp.append((y, k))
                        k += 1
                        j += 1
                    i += 1
                tsilen = i
                tsitotallen = j
                pposs = []
                surf_of_ans = []
            else:
                tsitotallen = 0

            rows_p_t = int(split[1] / fh)
            surfs = []
            # Needs commenting
            icount = 0
            for i in range(int(math.ceil(float(tslen + tsitotallen) / rows_p_t))):
                surf = pygame.Surface(split).convert_alpha()
                surf.fill(pygame.Color(0, 0, 0, 0))
                for j in range(rows_p_t):
                    n = i * rows_p_t + j
                    if n >= tslen:
                        if indent_text is None:
                            break
                        else:
                            n -= tslen
                            if n >= tsitotallen:
                                break
                            else:
                                tsimp = tsisimp[n]
                                if tsimp[1] == 0: # First line of an answer
                                    pposs.append((0, fh * j))
                                    if j == 0:
                                        ni = i - 1
                                    else:
                                        ni = i
                                    surf_of_ans.append((icount, ni))
                                    icount = i
                                tsurf = self.font.render(
                                    tsimp[0], antialias, color)
                                surf.blit(tsurf, (indent_size[0], fh * j))
                    else:
                        tsurf = self.font.render(ts[n], antialias, color)
                        surf.blit(tsurf, (0, fh * j))
                surfs.append(surf)
            if not indent_text:
                return surfs
            else:
                del surf_of_ans[0]
                surf_of_ans.append((icount, i))
                return surfs, pposs, surf_of_ans



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Dililatum: a quest system for simple RPGs
# Copyright (C) 2010  Niels Serup

# This file is part of Dililatum.
#
# Dililatum is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Dililatum is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Dililatum.  If not, see <http://www.gnu.org/licenses/>.

##[ Name        ]## statusprinter
##[ Maintainer  ]## Niels Serup <ns@metanohi.org>
##[ Description ]## Prints statuses with different prefixes

from datetime import datetime
import sys
try:
    from termcolor import colored
    def withcolor(msg, fg, bg):
        nmsg = ''
        spl = msg.split('\n')
        while spl[-1] == '':
            spl.pop()
        nmsg = colored(spl[0], fg, 'on_' + bg, attrs=['bold']) + '\n'
        nmsg += '\n'.join(spl[1:]) + '\n'

        return nmsg + '\n'
except Exception:
    def withcolor(msg, fg, bg):
        return msg + '\n'

def atleast(m, r):
    if r < m:
        return m
    else:
        return r

class StatusPrinter:
    def __init__(self, name, etc, fgcolor='black', bgcolor='white',
                 outfile=sys.stdout):
        self.etc = etc
        self.fgcolor = fgcolor
        self.bgcolor = bgcolor
        self.outfile = outfile
        self.header = '####' + name[:10] + \
            atleast(0, 10 - len(name) + 4) * '#' + '%s####'
        self.simpleheader = name + ':%s:'

    def __call__(self, msg):
        if self.etc.simpleprint:
            msg = self.simpleheader % datetime.now() + '\n' + msg + '\n'
            self.outfile.write(msg)
        else:
            msg = self.header % datetime.now() + '\n' + msg + '\n'
            if self.etc.verbose:
                if self.etc.colorprint:
                    self.outfile.write(withcolor(msg, self.fgcolor,
                                                 self.bgcolor))
                else:
                    self.outfile.write(msg + '\n')




#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Dililatum: a quest system for simple RPGs
# Copyright (C) 2010  Niels Serup

# This file is part of Dililatum.
#
# Dililatum is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Dililatum is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Dililatum.  If not, see <http://www.gnu.org/licenses/>.

##[ Name        ]## system
##[ Maintainer  ]## Niels Serup <ns@metanohi.org>
##[ Description ]## Controls the general aspects of the engine

import sys
import os
from datetime import datetime
from pygame.locals import MOUSEMOTION, MOUSEBUTTONUP, MOUSEBUTTONDOWN
import dililatum.various as various
from dililatum.statusprinter import StatusPrinter


class Event:
    def __init__(self, name='', args=[], uargs=[]):
        self.name = name
        self.args = args
        self.uargs = uargs

    def __str__(self):
        return 'event:%s;args=%s;uargs=%s' % (self.name,
                                              str(self.args),
                                              str(self.uargs))

class SignalDict(dict):
    def add(self, signal, *func_and_args):
        try:
            self.__getitem__(signal).append(func_and_args)
        except KeyError:
            self.__setitem__(signal, [func_and_args])

    def remove(self, signal, func):
        d = self.__getitem__(signal)
        for x in d:
            if x[0] == func:
                d.remove(x)
                break

    def run(self, signal, obj):
        try:
            for func in self.__getitem__(signal)[:]:
                try:
                    obj.uargs = func[1:]
                except Exception:
                    pass
                func[0](obj)
        except KeyError:
            pass

    def clear(self, signal):
        self.__setitem__(signal, [])

class System:
    def __init__(self, etc, error=None):
        self.etc = etc
        if error is None:
            self.error = various.usable_error
        else:
            self.error = error
        self.signalactions = SignalDict()
        self.gameactions = SignalDict()

        self.status = StatusPrinter('SYSTEM', self.etc, 'white', 'red')
        self.status('''\
Dililatum is free software: you are free to change and redistribute it
under the terms of the GNU GPL, version 3 or any later version.
There is NO WARRANTY, to the extent permitted by law. See
<http://metanohi.org/projects/dililatum/> for downloads and documentation.''')

        for f in self.etc.debugfiles:
            d = os.path.dirname(f)
            b = os.path.basename(f)
            b = b[:b.find('.py')]
            if os.path.isdir(d):
                sys.path.append(d)
                module_name = os.path.split(d)[1]
                if module_name == '':
                    module_name = d
            else:
                module_name = d
            mod = __import__(module_name + '.' + b, globals(), locals(),
                             ['main'], -1)
            mod.action = self.signalactions.add
            mod.main()
        self.emit_signal('aftersysteminit', self)

    def emit_signal(self, *signal_and_args):
        signal = signal_and_args[0]
        args = signal_and_args[1:]
        event = Event(signal, args)
        self.signalactions.run(signal, event)

    def emit_event(self, *event_and_type):
        if self.etc.zoom != 1 and \
                event_and_type[0] in (MOUSEMOTION, MOUSEBUTTONUP,
                                      MOUSEBUTTONDOWN):
            event_and_type = list(event_and_type[:])
            event_and_type[1] = [x / self.etc.zoom for x in event_and_type[1].pos]
        self.gameactions.run(*event_and_type)

    def start(self):
        self.emit_signal('beforesystemstart', self)
        self.status('Starting system...')

        if os.path.isdir(self.etc.game):
            if not os.path.isfile(os.path.join(self.etc.game,
                                               'game.py')):
                # If your game name is "SuperSheep", your package name
                # is likely to be "supersheep".
                pathdir = os.path.join(
                    self.etc.game,
                    os.path.split(os.path.abspath(self.etc.game))[1].lower())
            else:
                pathdir = self.etc.game
            path = os.path.split(os.path.abspath(pathdir))
            sys.path.insert(0, path[0])
            module_name = path[1]
            if module_name == '':
                module_name = self.etc.game
        else:
            module_name = self.etc.game

        fgame = __import__(module_name + '.game', globals(), locals(),
                           ['Game'], -1)
        self.game = fgame.Game(self)
        paths = []
        paths.append(os.path.join(os.path.dirname(fgame.__file__),
                                  self.game.datadir))
        paths.append('/usr/local/share/forestquest/data')
        paths.append('/usr/share/forestquest/data')
        for x in paths:
            if os.path.isdir(x):
                os.chdir(x)
                break
        self.status(
            'Name of game is: %s' % self.game.name + '\n' +
            'Size of game is: %dx%d' % tuple(self.game.size))
        self.emit_signal('aftersystemstart', self)
        self.status('Starting game...')
        self.game.start()
        self.status('Running game...')
        self.game.run()

    def end(self):
        self.emit_signal('beforesystemend', self)
        self.status('Stopping game...')
        self.game.end()
        self.emit_signal('aftersystemend', self)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Dililatum: a quest system for simple RPGs
# Copyright (C) 2010  Niels Serup

# This file is part of Dililatum.
#
# Dililatum is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Dililatum is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Dililatum.  If not, see <http://www.gnu.org/licenses/>.

##[ Name        ]## object
##[ Maintainer  ]## Niels Serup <ns@metanohi.org>
##[ Description ]## Contains the Object class whose purpose is to be
                  # drawn onto instances of Place classes (and thereby
                  # the screen)

import pygame
from pygame.locals import *

class Object:
    def __init__(self, world, **oargs):
        def get(key, default):
            try: return oargs[key]
            except KeyError: return default

        self.world = world
        self.rel = get('rel', 0) # 1 for real position, 0 for
                                 # modified (visible) position
        self.action = get('action', None)
        self.unaction = get('unaction', None)
        self.visible = get('visible', True)
        self.pos = get('pos', (0, 0))
        self.size = get('size', None)
        self.area = get('area', None)
        self.areaadd = get('areaadd', (0, 0))
        self.in_area = False
        if self.size is None and self.pos is not None and \
                'append' in dir(self.pos[0]) and len(self.pos) > 1:
            self.size = self.pos[1]
            self.pos = self.pos[0]
        self.imgfile = get('imgfile', None)
        if self.imgfile is not None and self.world.sys.etc.loadwait:
            self.load_image()
        else:
            self.surf = None
        self.walkonact = get('walkonact', True)
        self.require_event = get('require', None)
        if self.require_event is not None:
            self.world.link_event(self.require_event[0],
                                  self.event_check)
            self.close_to_touch = False

    def get_bottom_area(self):
        if self.surf is None: self.load_image()
        try:
            if self.rel == 0:
                return [a[1] + 0 for a in self.area]
            else:
                return [self.world.current_place.char_pos(self.area[1])[i]
                        + 0 for i in range(2)]
        except Exception:
            return 0, 0

    def load_image(self):
        if self.imgfile is None: return
        self.surf = self.world.load_image(self.imgfile, True)
        self.size = self.surf.get_size()
        if self.area is None:
            self.area = ([self.pos[i] + self.areaadd[i]
                          for i in range(2)],
                         [self.pos[i] + self.size[i]
                          for i in range(2)])
        else:
            for i in range(2):
                if self.area[0][i] is None:
                    self.area[0][i] = self.pos[i]
                if self.area[1][i] is None:
                    self.area[1][i] = self.area[0][i] + self.size[i]
                else:
                    self.area[1][i] -= self.area[0][i]

    def event_check(self, event):
        if self.world.keys_locked:
            return
        if event.key == self.require_event[1] and self.close_to_touch:
            self.actions(True)

    def check_if_action_needed(self, pos, size):
        if self.surf is None: self.load_image()
        retval = True

        if self.rel == 0:
            npos = self.world.current_place.char_pos(pos)
        else:
            npos = pos

        touch = self.feet_in_obj_check(self.area[0], self.area[1],
                                       pos, size)
        self.close_to_touch = touch and self.require_event is not None
        if touch:
            retval = self.walkonact

        if self.require_event is None:
            rtouch = touch
        else:
            rtouch = False
        self.actions(rtouch)
        return retval

    def actions(self, touch):
        if touch and not self.in_area:
            if self.require_event is None:
                self.in_area = True
            self.do_action()
        elif not touch and self.in_area:
            if self.require_event is None:
                self.in_area = False
            self.do_unaction()

    def feet_in_obj_check(self, spos, spos2, cpos, csize):
        pos1 = [cpos[0] - csize[0] / 2, cpos[1] - csize[1] / 20]
        pos2 = [cpos[0] + csize[0] / 2, cpos[1]]
        xos1 = spos
        xos2 = spos2

        return pos1[0] < xos2[0] and pos2[0] > xos1[0] and \
            pos1[1] < xos2[1] and pos2[1] > xos1[1]

    def do_action(self):
        if self.action is not None:
            self.action[0](*self.action[1:])

    def do_unaction(self):
        if self.unaction is not None:
            self.unaction[0](*self.unaction[1:])

    def show(self):
        self.visible = True

    def hide(self):
        self.visible = False

    def draw(self, surf=None):
        if not self.visible: return
        if self.surf is None: self.load_image()
        if self.surf is None: return
        if surf is None:
            surf = self.world

        surf.blit(self.surf, self.pos)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Dililatum: a quest system for simple RPGs
# Copyright (C) 2010  Niels Serup

# This file is part of Dililatum.
#
# Dililatum is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Dililatum is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Dililatum.  If not, see <http://www.gnu.org/licenses/>.

##[ Name        ]## walktest
##[ Maintainer  ]## Niels Serup <ns@metanohi.org>
##[ Description ]## Contains walk tests

from datetime import datetime
import os.path
import pygame
from pygame.locals import *


class WalkTestSimple:
    def __init__(self, files):
        self.files = files

    def start(self):
        pygame.display.init()

        self.screen = pygame.display.set_mode((300, 600))

        self.images = []
        for f in self.files:
            img = pygame.image.load(f)
            self.images.append((img.convert_alpha(), img.get_width(), img.get_height()))

        self.background = pygame.Surface(self.screen.get_size()).convert()

        self.clock = pygame.time.Clock()
        self.loop_images()

    def microseconds(self, tdelta):
        #return tdelta.microseconds + tdelta.seconds * 1000000 + \
        #    tdelta.min * 60000000
        return tdelta.microseconds

    def loop_images(self):
        done = False
        l = len(self.images)
        i = 0
        cr = 255
        cg = 0
        cb = 0

        time = datetime.now()
        while not done:
            self.clock.tick(30)
            #cr = 0
            #cg = 0
            #cb = 0
            self.background.fill((cr, cg, cb))
            self.screen.blit(self.background, (0, 0))
            img = self.images[i]
            self.screen.blit(img[0], ((300 - img[1]) / 2,
                                      (600 - img[2]) / 2))

            pygame.display.flip()
            if cr == 255 and cg < 255 and cb == 0:
                cg += 5
            elif cr > 0 and cg == 255 and cb == 0:
                cr -= 5
            elif cr == 0 and cg == 255 and cb < 255:
                cb += 5
            elif cr == 0 and cg > 0 and cb == 255:
                cg -= 5
            elif cr < 255 and cg == 0 and cb == 255:
                cr += 5
            elif cr == 255 and cg == 0 and cb > 0:
                cb -= 5
            if cr > 255: cr = 255
            elif cr < 0: cr = 0
            if cg > 255: cg = 255
            elif cg < 0: cg = 0
            if cb > 255: cb = 255
            elif cb < 0: cb = 0

            for event in pygame.event.get():
                if event.type == QUIT:
                    done = True
                elif event.type == KEYDOWN:
                    if event.key == K_ESCAPE:
                        done = True
            if self.microseconds(datetime.now() - time) > 250000:
                i = (i + 1) % l
                time = datetime.now()


class WalkTestAdvanced:
    def __init__(self, sett):
        self.directory = sett.directory
        self.time = sett.time
        self.direction = sett.direction
        self.bgtype = sett.background
        self.shadow = sett.shadow
        self.size = (sett.width, sett.height)

        self.frames = {}

    def file2surface(self, dirname, files):
        for i in range(len(files)):
            img = pygame.image.load(os.path.join(self.directory,
                                                 dirname, files[i]))
            files[i] = (img.convert_alpha(),
                        img.get_width(),
                        img.get_height())
        return files

    def path_walk(self, name, dirname, files):
        self.frames[name] = self.file2surface(dirname, sorted(files))

    def start(self):
        pygame.display.init()

        self.screen = pygame.display.set_mode(self.size)

        for x in 'lt', 'ct', 'rt', 'lm', 'rm', 'lb', 'cb', 'rb':
            os.path.walk(os.path.join(self.directory, x), self.path_walk, x)

        self.bgsurface = \
            pygame.Surface(self.screen.get_size()).convert()

        if self.shadow:
            self.create_shadow()
        else:
            self.shadow = None

        self.clock = pygame.time.Clock()
        self.loop()

    def create_shadow(self):
        size = (self.size[0] / 2, self.size[1] / 15)
        ssize = [s * 3 for s in size]
        rect = pygame.Rect((0, 0), ssize)
        surf = pygame.Surface(ssize).convert_alpha()
        surf.fill(pygame.Color(0, 0, 0, 0))
        pygame.draw.ellipse(surf, pygame.Color(0, 0, 0, 200), rect)
        surf = pygame.transform.smoothscale(surf, size)

        self.shadow = surf
        self.shadow_pos = (self.size[0] - size[0]) / 2

    def microseconds(self, tdelta):
        return tdelta.microseconds

    def loop(self):
        done = False
        i = 0
        cflow = self.bgtype == 'colorflow'
        if cflow:
            cr = 255
            cg = 0
            cb = 0
        else:
            color = self.bgtype

        time = datetime.now()
        while not done:
            self.clock.tick(30)
            if cflow:
                color = (cr, cg, cb)
            self.bgsurface.fill(color)
            self.screen.blit(self.bgsurface, (0, 0))

            if i >= len(self.frames[self.direction]):
                i = 0
            img = self.frames[self.direction][i]

            if self.shadow is not None:
                self.screen.blit(self.shadow, (self.shadow_pos,
                (self.size[1] - img[2]) / 2 + img[2] - img[2] / 10))

            self.screen.blit(img[0], ((300 - img[1]) / 2,
                                      (600 - img[2]) / 2))

            pygame.display.flip()
            if cflow:
                if cr == 255 and cg < 255 and cb == 0:
                    cg += 5
                elif cr > 0 and cg == 255 and cb == 0:
                    cr -= 5
                elif cr == 0 and cg == 255 and cb < 255:
                    cb += 5
                elif cr == 0 and cg > 0 and cb == 255:
                    cg -= 5
                elif cr < 255 and cg == 0 and cb == 255:
                    cr += 5
                elif cr == 255 and cg == 0 and cb > 0:
                    cb -= 5
                if cr > 255: cr = 255
                elif cr < 0: cr = 0
                if cg > 255: cg = 255
                elif cg < 0: cg = 0
                if cb > 255: cb = 255
                elif cb < 0: cb = 0

            for event in pygame.event.get():
                if event.type == QUIT:
                    done = True
                elif event.type == KEYDOWN:
                    if event.key == K_ESCAPE:
                        done = True
                    elif event.key == K_KP7 or event.key == K_7:
                        self.direction = 'lt'
                    elif event.key == K_KP8 or event.key == K_8:
                        self.direction = 'ct'
                    elif event.key == K_KP9 or event.key == K_9:
                        self.direction = 'rt'
                    elif event.key == K_KP4 or event.key == K_4:
                        self.direction = 'lm'
                    elif event.key == K_KP6 or event.key == K_6:
                        self.direction = 'rm'
                    elif event.key == K_KP1 or event.key == K_1:
                        self.direction = 'lb'
                    elif event.key == K_KP2 or event.key == K_2:
                        self.direction = 'cb'
                    elif event.key == K_KP3 or event.key == K_3:
                        self.direction = 'rb'

            if self.microseconds(datetime.now() - time) > self.time:
                i = (i + 1) % len(self.frames[self.direction])
                time = datetime.now()




def extra1(event):
    print event.args[0].etc.arguments

def extra2(event):
    print event

def extra3(event):
    char = event.args[0]
    verb = event.uargs[0]
    print char.id + ' ' + verb + ' the name..'

def main():
    action('beforesystemstart', extra1)
    action('beforegamestart', extra2)
    action('beforecharacteradd', extra3, 'is')



version = '0.1'
version_text = '''\
Dililatum 0.1
Copyright (C) 2010  Niels Serup
License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>
This is free software: you are free to change and redistribute it.
There is NO WARRANTY, to the extent permitted by law.\
'''



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Dililatum: a quest system for simple RPGs
# Copyright (C) 2010  Niels Serup

# This file is part of Dililatum.
#
# Dililatum is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Dililatum is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Dililatum.  If not, see <http://www.gnu.org/licenses/>.

##[ Name        ]## game
##[ Maintainer  ]## Niels Serup <ns@metanohi.org>
##[ Description ]## Contains the generic form of the Game class

from datetime import datetime
import os
from dililatum.world import World
from dililatum.statusprinter import StatusPrinter
from dililatum.various import *

class SearchDict(dict):
    def __getitem__(self, key):
        p = []
        for k, v in self.items():
            if k.find(key) != -1:
                p.append(v)
        if len(p) == 0:
            return None
        elif len(p) == 1:
            return p[0]
        else:
            return p

    def precise_get(self, key):
        return dict.__getitem__(self, key)

class GenericGame:
    name = 'A game'
    shortname = 'game'
    size = (640, 480)
    datadir = 'data/'

    def __init__(self, stm):
        self.sys = stm
        self.status = StatusPrinter(self.shortname.upper(),
                                    self.sys.etc, 'green', 'grey')
        self.sys.emit_signal('aftergameinit', self)

    def create_world(self):
        self.world = World(self.sys, self.size)

    def get_path_data(self, path):
        paths = {}
        for root, dirs, files in os.walk(path):
            name = root[len(path)+1:]
            if name == '':
                name = '.'
            paths[name] = dirs, sorted(files)
        return path, paths

    def start(self):
        self.sys.emit_signal('beforegamestart', self)
        self.start_game()
        self.sys.emit_signal('aftergamestart', self)

    def start_game(self):
        pass

    def run(self):
        self.sys.emit_signal('beforegamerun', self)
        self.run_game()
        self.sys.emit_signal('aftergamerun', self)

    def run_game(self):
        pass

    def load_sounds(self, dirpath, add=True):
        snds = SearchDict()
        f = self.get_path_data(dirpath)[1]['.'][1]
        for x in f:
            snd = self.load_sound(os.path.join(dirpath, x), add)
            snds[x] = snd
        return snds

    def load_sound(self, path, add=True):
        snd = self.world.create_sound(path)
        if add:
            self.world.add_sound(snd)
        return snd

    def load_places(self, imgpath, posokpath, objspath, num=None):
        imgpath = os.path.normpath(imgpath)
        posokpath = os.path.normpath(posokpath)
        objspath = os.path.normpath(objspath)
        if num is None:
            num = min(
                len(self.get_path_data(imgpath)[1]['.'][1]),
                len(self.get_path_data(posokpath)[1]['.'][1]),
            )
        numm = num - 1
        pref = '%0' + str(len(str(numm))) + 'd'

        self.sys.emit_signal('beforeplacesload', self, numm, pref % numm)
 
        overlays = {}
        for x in self.get_path_data(objspath)[1]['.'][1]:
            i = x.split('-')
            inum = i[0]
            pos = [int(y) for y in i[1].split(',')]
            rel = int(i[2])
            spl = i[3].split(',')
            for y in range(2):
                try:
                    spl[y] = int(spl[y])
                except Exception:
                    spl[y] = None
            end_or_start = spl
            if len(i) < 6:
                start = [None, 0]
                end = end_or_start
                name = i[4].split('.')[0]
            else:
                start = end_or_start
                end = [int(y) for y in i[3].split(',')]
                name = i[5].split('.')[0]
            inf = (x, pos, rel, start, end, name)
            try:
                overlays[inum].append(inf)
            except Exception:
                overlays[inum] = [inf]

        for i in range(num):
            prefi = pref % i
            self.sys.emit_signal('beforeplaceload', self, i, prefi)
            okbg = False
            suffs = ['jpg', 'JPG', 'jpeg', 'JPEG', 'png', 'PNG', \
                'gif', 'GIF', 'tga', 'TGA']
            while not okbg:
                suff = suffs[0]
                del suffs[0]
                backgroundpath = os.path.join(imgpath, prefi + '.' + suff)
                okbg = os.path.isfile(backgroundpath)
                if not okbg and len(suffs) == 0:
                    break
            if not okbg:
                self.sys.error('cannot find image for place ' + prefi)
                continue

            okpospath = os.path.join(posokpath, prefi + '.dililatumpos')
            place = self.world.create_place(
                backgroundpath,
                okpospath
            )
            try:
                ovobjs = overlays[prefi]
            except KeyError:
                ovobjs = []
            for i in range(len(ovobjs)):
                c = ovobjs[i]
                ovobjs[i] = self.world.create_object(
                    rel=c[2], pos=c[1], area=(c[3], c[4]),
                    imgfile=os.path.join(objspath, c[0]),
                    )
                #ovobjs[i].action = [ovobjs[i].show]
                #ovobjs[i].unaction = [ovobjs[i].hide]
                place.add_object(ovobjs[i])
                place.obj_names[c[5]] = ovobjs[i]
            self.world.add_place(place)
            self.sys.emit_signal('afterplaceload', self, i, prefi)
        self.sys.emit_signal('afterplacesload', self)

    def use_data_from_map(self, path):
        plcmap = self.load_map_data(path)
        self.use_map_data(plcmap)

    def load_map_data(self, detailsfile):
        self.sys.emit_signal('beforemapload', self)
        det = read_file(detailsfile)
        dirs = {
            '^': 'up',
            '>': 'right',
            'v': 'down',
            '<': 'left'
        }
        spl = det.split('\n\n')
        info = []
        for i in range(len(spl)):
            info.append(None)
        for x in spl:
            t = x.split('\n')
            num = int(t[0])
            sinfo = {}
            for y in t[1:]:
                if y == '': continue
                tdet = y.split('|')
                try:
                    direction = dirs[tdet[0]]
                except KeyError:
                    direction = tdet[0]

                try:
                    target = int(tdet[1])
                except Exception:
                    target = None
                tdett = tdet[3].split(' ')
                objpos = eval('[' + tdett[0] + ']')
                objsize = eval('[' + tdett[1] + ']')
                try:
                    targetpos = eval('[' + tdet[2] + ']')
                except Exception:
                    targetpos = None
                targetdir = tdet[4]
                sinfo[direction] = (target, objpos, objsize,
                                    targetpos, targetdir)
            try:
                info[num] = sinfo
            except IndexError:
                pass

        return info

    def use_map_data(self, data):
        places = self.world.places
        i = 0
        for x in data:
            if x is None: continue
            for key, vals in x.items():
                if vals[0] is None or vals[3] is None or vals[4] == '':
                    actwhat = None
                else:
                    actwhat = [self.world.set_place, places[vals[0]], vals[3], vals[4]]
                obj = self.world.create_object(
                    pos=vals[1], size=vals[2], area=vals[1:3],
                    action=actwhat, walkonact=False)
                obj.isdirobj = True
                places[i].add_object(obj)
                if key:
                    places[i].set_direction_object(key, obj)
            i += 1

        self.sys.emit_signal('aftermapload', self)

    def fix_places_with_minus_x_positions(self):
        places = self.world.places
        for p in places:
            for o in p.objects:
                if not 'isdirobj' in dir(o) or o.action is None:
                    continue
                targetpos = o.action[2]
                targetdir = o.action[3]
                if targetpos[0] < 0:
                    targetpos[0] = int(
                        self.size[0] + targetpos[0] -
                        o.action[1].char_size(targetpos) *
                        self.world.leading_character.get_frame().width
                    )
                elif targetdir[0] == 'l':
                    targetpos[0] = int(
                        targetpos[0] -
                        o.action[1].char_size(targetpos) *
                        self.world.leading_character.get_frame().width
                    )
                elif targetdir in ('ct', 'cb', 'lt', 'lb', 'rt', 'rb'):
                    targetpos[0] = int(
                        targetpos[0] -
                        o.action[1].char_size(targetpos) / 2 *
                        self.world.leading_character.get_frame().width
                    )

    def load_character(self, path, add=True, **oargs):
        char = self.world.create_character(
            path, self.get_path_data(path), **oargs)
        if add:
            self.world.add_character(char)
        return char

    def load_msgbox(self, **oargs):
        msgbox = self.world.create_msgbox(**oargs)
        self.world.add_msgbox(msgbox)
        return msgbox

    def load_object(self, **oargs):
        obj = self.world.create_object(**oargs)
        return obj

    def load(self):
        pass

    def save(self):
        pass

    def end(self):
        self.sys.emit_signal('beforegameend', self)
        self.world.end()
        self.sys.emit_signal('aftergameend', self)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Dililatum: a quest system for simple RPGs
# Copyright (C) 2010  Niels Serup

# This file is part of Dililatum.
#
# Dililatum is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Dililatum is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Dililatum.  If not, see <http://www.gnu.org/licenses/>.

##[ Name        ]## sound
##[ Maintainer  ]## Niels Serup <ns@metanohi.org>
##[ Description ]## Contains the Sound class, which makes it possible
                  # to play back sound

import pygame
from pygame.locals import *

class Sound:
    def __init__(self, world, path):
        self.world = world
        self.path = path
        self.is_playing = False
        if self.world.sys.etc.loadwait:
            self.load_sound()
        else:
            self.snd = None

    def load_sound(self):
        self.snd = pygame.mixer.Sound(self.path)

    def play(self):
        if self.world.sys.etc.mute: return
        self.world.sys.emit_signal('beforesoundplay', self, self.snd)
        if self.snd is None:
            self.load_sound()
        self.snd.play(-1) # Looping indefinitely
        self.is_playing = True
        self.world.sys.emit_signal('aftersoundplay', self, self.snd)

    def stop(self):
        self.world.sys.emit_signal('beforesoundstop', self, self.snd)
        try:
            self.snd.stop()
        except AttributeError:
            pass
        self.is_playing = False
        self.world.sys.emit_signal('aftersoundstop', self, self.snd)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Dililatum: a quest system for simple RPGs
# Copyright (C) 2010  Niels Serup

# This file is part of Dililatum.
#
# Dililatum is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Dililatum is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Dililatum.  If not, see <http://www.gnu.org/licenses/>.

##[ Name        ]## posmarker
##[ Maintainer  ]## Niels Serup <ns@metanohi.org>
##[ Description ]## Contains the OKPositionsMarker class to aid in
                  # marking where on a background characters can
                  # safely walk

import sys
import pygame
from pygame.locals import *
from dililatum.bitmap import BitMap

class NewSurface(pygame.Surface):
    def __init__(self, *args):
        pygame.Surface.__init__(self, *args)
        self.x = args[0][0]
        self.y = args[0][1]
        self.none = pygame.Color(0, 0, 0, 0)
        self.color = pygame.Color(18, 13, 228, 90)
        self.fill(self.none)

    def set_checkpoints(self, pos):
        self.positions = pos

    def set_at(self, x, y):
        pygame.Surface.set_at(self, (x, y), self.color)

    def del_at(self, x, y):
        pygame.Surface.set_at(self, (x, y), self.none)

    def add_rect(self, pos, size):
        s2 = size / 2
        x1 = pos[0] - s2
        y1 = pos[1] - s2
        x2 = pos[0] + s2
        y2 = pos[1] + s2
        if x1 < 0: x1 = 0
        if y1 < 0: y1 = 0

        for i in range(y1, y2):
            if i >= self.y: break
            for j in range(x1, x2):
                if j >= self.x: break
                if not self.positions.get(j, i):
                    self.set_at(j, i)
                    self.positions.set(j, i)

    def del_rect(self, pos, size):
        s2 = size / 2
        x1 = pos[0] - s2
        y1 = pos[1] - s2
        x2 = pos[0] + s2
        y2 = pos[1] + s2

        for i in range(y1, y2):
            for j in range(x1, x2):
                self.del_at(j, i)
                self.positions.unset(j, i)

class OKPositionsMarker:
    def __init__(self, imgfile, posfile):
        self.imgfile = imgfile
        self.posfile = posfile

        self.rect_size = 50

    def start(self):
        pygame.display.init()
        self.screen = pygame.display.set_mode((640, 480))
        pygame.display.set_caption('posmarker')
        self.img = pygame.image.load(self.imgfile).convert()
        self.points = NewSurface((640, 480), SRCALPHA)
        self.positions = BitMap(640, 480)
        self.points.set_checkpoints(self.positions)

        try:
            self.positions.load(self.posfile)
            for i in xrange(480):
                for j in xrange(640):
                    if self.positions.get(j, i):
                        self.points.set_at(j, i)
        except Exception:
            pass

        self.clock = pygame.time.Clock()
        self.loop()

    def end(self):
        self.positions.arr.tofile(self.posfile)

    def loop(self):
        done = False
        pressed = 0, 0, 0
        while not done:
            self.clock.tick(30)

            for event in pygame.event.get():
                if event.type == MOUSEBUTTONDOWN or event.type == MOUSEBUTTONUP:
                    pressed = pygame.mouse.get_pressed()
                elif event.type == KEYDOWN:
                    if event.key == K_PAGEUP:
                        self.rect_size += 1
                        if self.rect_size > 100:
                            self.rect_size = 100
                    elif event.key == K_PAGEDOWN:
                        self.rect_size -= 1
                        if self.rect_size < 1:
                            self.rect_size = 1
                    elif event.key == K_ESCAPE:
                        done = True
                elif event.type == QUIT:
                    done = True
            point = pygame.mouse.get_pos()
            if pressed[0]:
                self.points.add_rect(point, self.rect_size)
            elif pressed[2]:
                self.points.del_rect(point, self.rect_size)

            self.screen.blit(self.img, (0, 0))
            self.screen.blit(self.points, (0, 0))
            pygame.display.flip()



#!/usr/bin/env python
from distutils.core import setup
import os
ginfo_file = os.path.join(os.path.dirname(os.path.realpath(__file__)),
                          'dililatum', 'generalinformation.py')
execfile(ginfo_file) # Gives us a version and a version_text variables
bins = [x[2] for x in os.walk('bin')][0]
bins = [os.path.join('bin', x) for x in bins]

setup(
    name='Dililatum',
    version=version,
    author='Niels Serup',
    author_email='ns@metanohi.org',
    packages=['dililatum', 'dililatum.tools'],
    scripts=bins,
    url='http://metanohi.org/projects/dililatum/',
    license='LICENSE.txt',
    description='A quest system for simple RPGs',
    long_description=open('README.txt').read(),
    classifiers=["Development Status :: 3 - Alpha",
                 "Intended Audience :: End Users/Desktop",
                 "Intended Audience :: Developers",
                 "License :: OSI Approved :: GNU General Public License (GPL)",
                 "Operating System :: OS Independent",
                 "Programming Language :: Python",
                 "Topic :: Games/Entertainment :: Role-Playing",
                 "Topic :: Software Development :: Libraries :: pygame",
                 "Topic :: Software Development :: Libraries :: Python Modules",
                 "Topic :: Utilities"
                 ],
    requires=['pygame', 'numpy']
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Dililatum: a quest system for simple RPGs
# Copyright (C) 2010  Niels Serup

# This file is part of Dililatum.
#
# Dililatum is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Dililatum is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Dililatum.  If not, see <http://www.gnu.org/licenses/>.

##[ Name        ]## bitmap
##[ Maintainer  ]## Niels Serup <ns@metanohi.org>
##[ Description ]## Contains the BitMap class which enable bits to be
                  # stored as bits and not bytes

import numpy

class BitMap:
    def __init__(self, x, y):
        self.x = x
        self.y = y
        self.arr = numpy.zeros((x, y/8), dtype=numpy.int8)

    def load(self, filename):
        f = open(filename, 'rb')
        c = f.read()
        p = 0
        for j in range(self.x):
            for i in range(self.y/8):
                self.arr[j][i] = ord(c[p])
                p += 1
        f.close()

    def set(self, x, y, do=True):
        try:
            p = self.arr[x][y/8]
        except IndexError:
            return
        tmp = 1;
        for i in range(y % 8):
            tmp *= 2
        if do and not self.get(x, y):
            self.arr[x][y/8] += tmp
        elif not do and self.get(x, y):
            self.arr[x][y/8] -= tmp

    def unset(self, x, y):
        self.set(x, y, False)

    def get(self, x, y):
        try:
            p = self.arr[x][y/8]
        except IndexError:
            return
        tmp = 1;
        for i in range(int(y % 8)):
            tmp *= 2
        return (p / tmp) % 2 == 1



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Dililatum: a quest system for simple RPGs
# Copyright (C) 2010  Niels Serup

# This file is part of Dililatum.
#
# Dililatum is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Dililatum is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Dililatum.  If not, see <http://www.gnu.org/licenses/>.

##[ Name        ]## placelinker
##[ Maintainer  ]## Niels Serup <ns@metanohi.org>
##[ Description ]## Makes it possible to visually link places together

import sys
import os.path
import math
import random
import pygame
from pygame.locals import *
try:
    import cPickle as pickle
except ImportError:
    import pickle


placelinker_help = """\
To use this tool, remember the following:

LMB down => move screen
RMB down => move current background
MMB down => center screen to current position
Mouse scrolling and PageDown/PageUp => zoom in/out

Special keys:
s => stop any current special key action
r => create a rectangle on a background
    how: press r, click on start pos, click on end pos, type name,
         press enter, return or escape, type target direction, press
         enter, return or escape
p => create a point on a background
    how: press p, click on a position
l => create a link (points can have more than one link, but rects cannot)
    how: press l, click on a rectangle or a point, do that again
n => name a rectangle
    how: press n, click on a rectangle, type name, press enter, return
         or escape
d => set the target direction
    how: press d, click on a rectangle, type target direction, press
         enter, return or escape
m => move an object on a background
    how: press m, mouse down on an object, move object, mouse up
a => adjust the size of a rectangle
    how: press a, click on a rectangle, click on the new start pos,
         click on the new end pos
shift+a => adjust the start pos of a rectangle
    how: press a, click on a rectangle, click on the new start pos,
ctrl+a => adjust the end pos of a rectangle
    how: press a, click on a rectangle, click on the new end pos
delete: delete an object or a background
    how: press delete, click on what you want to throw away
e: delete link associated with an object
    how: press e, click on an object\
"""


class LinkedPlace:
    def __init__(self, parent, surf, pos, path):
        self.parent = parent
        self.surf = surf
        self.size = surf.get_size()
        self.pos = pos
        self.path = path

        self.current = False

        self.rects = []
        self.points = []

        self.highlighted_obj = None

        self.borders = Borders(self)

        surf = self.parent.large_font.render(os.path.basename(path),
                                             True, (0, 0, 255))
        self.surf.blit(surf, (0, 0))

    def check_hovered(self, pos):
        ipos = [pos[i] - self.pos[i] for i in range(2)]
        h = ipos[0] >= 0 and ipos[0] < self.size[0] and \
            ipos[1] >= 0 and ipos[1] < self.size[1]

        self.current = h
        self.check_internal_hovered(ipos)
        return h

    def check_internal_hovered(self, rpos):
        h = False
        self.highlighted_obj = None
        for p in self.points:
            if rpos[0] >= p.rpos[0] and rpos[1] >= p.rpos[1] and \
                    rpos[0] < p.rpos[0] + p.size[0] and \
                    rpos[1] < p.rpos[1] + p.size[1]:
                p.highlight()
                self.highlighted_obj = p
                h = True
            elif p.highlighted:
                p.unhighlight()

        for r in self.rects:
            if rpos[0] >= r.rpos[0] and rpos[1] >= r.rpos[1] and \
                    rpos[0] < r.rpos[0] + r.size[0] and \
                    rpos[1] < r.rpos[1] + r.size[1]:
                if not h:
                    r.highlight()
                    self.highlighted_obj = r
            elif r.highlighted:
                r.unhighlight()

    def create_rect(self, rpos, size, name='', target=''):
        rect = Rect(self, rpos, size, name, target)
        self.rects.append(rect)
        return rect

    def create_point(self, rpos):
        point = Point(self, rpos)
        self.points.append(point)
        return point

    def remove_object(self, o):
        if o is not None:
            if o in self.points:
                self.points.remove(o)
            elif o in self.rects:
                self.rects.remove(o)
            else:
                return
        for l in o.linked_with:
            self.parent.links.remove(l)
            for w in l.objs:
                try:
                    w.linked_with.remove(l)
                except ValueError:
                    pass

    def delete_highlighted_object(self):
        o = self.highlighted_obj
        self.remove_object(o)
        self.highlighted_obj = None

    def delete_self(self):
        for x in self.rects:
            self.remove_object(x)
        for x in self.points:
            self.remove_object(x)

        self.parent.places.remove(self)
        self.highlighted_obj = None

    def blit(self, surf):
        surf.pos = [surf.rpos[i] + self.pos[i] for i in range(2)]
        self.parent.blit(surf)

    def draw(self):
        if self.current:
            self.borders.draw()
        self.parent.blit(self)
        for r in self.rects:
            r.draw()
        for p in self.points:
            p.draw()


class LinkedPlaceObject:
    var='obj'

    def __init__(self, place, *args):
        self.place = place
        self.highlighted = False
        self.linked_with = []
        self.init(*args)
        self.unmark()
        self.update()

    def init(self, *args):
        self.normal_color = pygame.Color(0, 0, 0)
        self.highlight_color = pygame.Color(255, 255, 255)

    def highlight(self):
        self.highlighted = True
        self.mark()
        self.update()

    def unhighlight(self):
        self.highlighted = False
        self.unmark()
        self.update()

    def mark(self):
        pass

    def unmark(self):
        pass

    def update(self):
        pass

    def draw(self):
        self.place.blit(self)

class Borders(LinkedPlaceObject):
    def init(self):
        self.rpos = [-s / 25 for s in self.place.size]
        self.size = [int(s + s / 12.5) for s in self.place.size]
        self.surf = pygame.Surface(self.size)
        self.surf.fill((0, 0, 0))

class Text:
    def __init__(self, rect, text, where, font, color):
        self.rect = rect
        self.text = text
        self.where = where
        self.font = font
        self.fgcolor = color

    def draw(self):
        if self.rect.name == '':
            return
        surf = self.font.render(self.text(), True, self.fgcolor)
        size = surf.get_size()
        ratio = 1
        maxsize = [s * 0.9 for s in self.rect.size]
        if size[0] > maxsize[0]:
            ratio = maxsize[0] / float(size[0])
        if size[1] > maxsize[1]:
            nratio = maxsize[1] / float(size[1])
            if nratio < ratio:
                ratio = nratio

        if ratio != 1:
            size = [int(s * ratio) for s in size]
            surf = pygame.transform.smoothscale(surf, size)
        if self.where[0] == 1 and self.where[1] == 1:
            pos = [(self.rect.size[i] - size[i]) / 2 for i in range(2)]
        elif self.where[0] == 2 and self.where[1] == 2:
            pos = [(self.rect.size[i] - size[i]) for i in range(2)]
        self.rect.surf.blit(surf, pos)

class Rect(LinkedPlaceObject):
    var = 'rect'
    def init(self, rpos, size, name='', targetdir=''):
        self.rpos = rpos
        self.size = size
        self.name = name
        self.targetdir = targetdir

        self.normal_color = pygame.Color(18, 13, 228, 70)
        self.highlight_color = pygame.Color(118, 213, 128, 90)
        self.text = Text(self, lambda: self.name, (1, 1),
                         self.place.parent.large_font, self.place.parent.large_font_color)
        self.targettext = Text(self, lambda: self.targetdir, (2, 2),
                               self.place.parent.small_font, self.place.parent.small_font_color)

    def recreate(self):
        self.surf = pygame.Surface(self.size).convert_alpha()

    def mark(self):
        self.recreate()
        self.surf.fill(self.highlight_color)

    def unmark(self):
        self.recreate()
        self.surf.fill(self.normal_color)

    def update(self):
        if self.highlighted:
            self.mark()
        else:
            self.unmark()
        self.text.draw()
        self.targettext.draw()

class Point(LinkedPlaceObject):
    var = 'point'
    def init(self, rrpos):
        self.rrpos = rrpos
        self.size = (20, 20)
        self.rpos = [self.rrpos[i] - self.size[i] / 2 for i in range(2)]

        self.surf = pygame.Surface(self.size)
        self.normal_color = pygame.Color(0, 0, 0)
        self.highlight_color = pygame.Color(255, 255, 255)

    def mark(self):
        self.surf.fill(self.highlight_color)

    def unmark(self):
        self.surf.fill(self.normal_color)

class Link:
    def __init__(self, parent, *objs):
        self.parent = parent
        self.objs = objs
        for o in self.objs:
            o.linked_with.append(self)

    def draw(self):
        self.parent.draw_line(*[[o.pos[i] + o.size[i] / 2
                                 for i in range(2)]
                                for o in self.objs])

class PlaceLinker:
    def __init__(self, size=None):
        self.size = size
        self.zoom = 1.0
        self.offset = [0, 0]
        self.places = []
        self.links = []

        self.bgcolor = pygame.Color(255, 255, 255)
        self.current_hovered_place = None
        self.moving = False
        self.moving_a_place = False
        self.moving_an_object = False

        self.creating_rect = False
        self.adjusting_rect = False
        self.creating_point = False
        self.creating_link = False
        self.naming_rect = False
        self.setting_target_dir = False
        self.deleting_obj = False
        self.deleting_link = False

        self.special_key_func = None

    def get_screen_size(self):
        try:
            info = pygame.display.Info()
            size = info.current_w, info.current_h
            if size[0] == -1: # in this case, size[1] will also be -1
                sys.stderr.write('your SDL is too old for width \
and height detection\n')
                sys.exit(1)
        except Exception:
            sys.stderr.write('your PyGame is too old for width \
and height detection\n')
            sys.exit(1)
        self.size = size

    def start(self):
        pygame.display.init()
        pygame.font.init()

        if self.size is None:
            self.get_screen_size()

        self.screen = pygame.display.set_mode(self.size)
        pygame.display.set_caption('placelinker')
        self.bgsurface = pygame.Surface(self.size)
        self.bgsurface.fill(self.bgcolor)

        match = pygame.font.match_font('freesans,dejavusans', True)
        self.large_font = pygame.font.Font(match, 30)
        self.large_font_color = pygame.Color(233, 30, 45)
        self.small_font = pygame.font.Font(match, 20)
        self.small_font_color = pygame.Color(30, 233, 45)

    def blit(self, obj):
        size = obj.size
        surf = obj.surf
        pos = obj.pos

        size = [int(s * self.zoom) for s in size]
        pos = [int((pos[i] + self.offset[i]) * self.zoom) for i in
               range(2)]

        if pos[0] + size[0] >= 0 and \
                pos[0] <= self.size[0] and \
                pos[1] + size[1] >=0 and \
                pos[1] <= self.size[1]:
            surf = pygame.transform.scale(surf, size)
            self.screen.blit(surf, pos)

    def draw_line(self, *p):
        p = [self.get_screen_pos(x) for x in p]
        pygame.draw.aaline(self.screen, (128, 55, 38), p[0], p[1])
        pygame.draw.aaline(self.screen, (128, 55, 38),
                           (p[0][0] - 1, p[0][1]),
                           (p[1][0] - 1, p[1][1]))
        pygame.draw.aaline(self.screen, (128, 55, 38),
                           (p[0][0] + 1, p[0][1]),
                           (p[1][0] + 1, p[1][1]))

    def new_place(self, surf, x, y, name):
        return LinkedPlace(self, surf, (x, y), name)

    def new(self, *images):
        num = int(math.ceil(math.sqrt(len(images))))
        x = 0
        y = 0
        xi = 0
        size = None
        for path in images:
            try:
                plc = self.new_place(pygame.image.load(path).convert(),
                                     x, y, path)
                self.places.append(plc)
                if size is None:
                    size = plc.size[:]
                xi = (xi + 1) % num
                if xi == 0:
                    x = 0
                    y += int(size[1] * 1.5)
                else:
                    x += int(size[0] * 1.5)
            except pygame.error, e:
                sys.stderr.write('failed loading %s, python said:\
 %s\n' % (path, str(e)))
        xmax = size[0] * num + (size[0] / 2) * (num - 1)
        ymax = size[1] * num + (size[1] / 2) * (num - 1)

        self.zoom = self.size[0] / float(xmax) * 0.9
        self.offset[0] = (self.size[0] / self.zoom - xmax) / 2
        self.offset[1] = (self.size[1] / self.zoom - ymax) / 2

        self.outfilename = 'linked-places.lnkdplcs'
        self.fileoverride = False

    def load(self, datapath, *images):
        data = pickle.load(open(datapath, 'rb'))
        self.offset = list(data['offset'])
        self.zoom = data['zoom']
        for x in data['places']:
            plcinf = x[0]
            rects = x[1]
            points = x[2]
            plc = self.new_place(
                pygame.image.load(plcinf[0]).convert(),
                plcinf[1][0], plcinf[1][1], plcinf[0])
            self.places.append(plc)

            for r in rects:
                nrect = plc.create_rect(*r)
            for p in points:
                plc.create_point(p)

        for objnums in data['links']:
            objs = []
            for x in objnums:
                obj = None
                plcrange = range(len(self.places))
                for i in plcrange:
                    if i != x[1]:
                        continue

                    if x[0] == 1:
                        search_in = self.places[i].rects
                    elif x[0] == 2:
                        search_in = self.places[i].points
                    for j in range(len(search_in)):
                        if j == x[2]:
                            obj = search_in[j]
                            break
                    if obj is not None:
                        break
                objs.append(obj)
            self.create_link(*objs)

        self.outfilename = datapath
        self.fileoverride = True

        if len(images) > 0:
            for x in images:
                self.add_place(x)

    def add_place(self, path):
        plc = self.new_place(pygame.image.load(path).convert(),
                             0, 0, path)
        size = plc.size
        pos1 = [0, 0]

        ok = False
        while not ok:
            pos2 = [pos1[i] + size[i] for i in range(2)]

            nok = True
            for x in self.places:
                xos1 = x.pos
                xos2 = [xos1[i] + x.size[i] for i in range(2)]
                tok = (pos2[0] < xos1[0] or pos1[0] > xos2[0]) and \
                    (pos2[1] < xos1[1] or pos1[1] > xos2[1])
                if not tok:
                    nok = False
                    break

            if nok:
                ok = True
            else:
                pos1 = [pos1[i] + random.randint(10, 30) * size[i] / 100.0
                        for i in range(2)]

        if ok:
            plc.pos = pos1
            self.places.append(plc)

    def save(self):
        for p in self.places:
            for r in p.rects:
                if 0 in r.size:
                    p.remove_object(r)

        ofn = self.outfilename
        if not self.fileoverride:
            while os.path.exists(ofn):
                ofn = '_' + ofn

        data = {}
        data['zoom'] = self.zoom
        data['offset'] = tuple(self.offset)

        dp = []
        for plc in self.places:
            place = (plc.path, plc.pos)
            rects = []
            points = []
            for r in plc.rects:
                rects.append((r.rpos, r.size, r.name, r.targetdir))
            for p in plc.points:
                points.append(p.rrpos)
            dp.append((place, tuple(rects), tuple(points)))
        data['places'] = tuple(dp)

        dl = []
        plcrange = range(len(self.places))
        for l in self.links:
            objs = []
            for o in l.objs:
                objnum = None
                for i in plcrange:
                    tp = self.places[i]
                    for j in range(len(tp.rects)):
                        if tp.rects[j] == o:
                            objnum = (1, i, j)
                            break
                    for j in range(len(tp.points)):
                        if tp.points[j] == o:
                            objnum = (2, i, j)
                            break
                    if objnum is not None:
                        break
                objs.append(objnum)
            dl.append(tuple(objs))
        data['links'] = tuple(dl)

        try:
            f = open(ofn, 'wb')
            pickle.dump(data, f, pickle.HIGHEST_PROTOCOL)
            f.close()
        except Exception, e:
            sys.stderr.write(e + '\n\n\n')
            sys.stdout.write(pickle.dumps(data))

    def get_true_pos(self, pos):
        return [pos[i] / self.zoom - self.offset[i] for i in range(2)]

    def get_screen_pos(self, pos):
        return [(pos[i] + self.offset[i]) * self.zoom for i in range(2)]

    def relative_zoom(self, pos, way):
        opos = self.get_true_pos(pos)

        if way is True:
            zoom = self.zoom * 1.1
            if zoom > 10.0: zoom = 10.0
            self.zoom = zoom
        else:
            zoom = self.zoom / 1.1
            if zoom < 0.01: zoom = 0.01
            self.zoom = zoom

        self.offset = [pos[i] / self.zoom - opos[i] for i in range(2)]

        for p in self.places:
            for r in p.rects:
                r.update()

        if self.moving_an_object:
            self.end_obj_moving()
        if self.moving:
            self.end_screen_moving()
        if self.moving_a_place:
            self.end_place_moving()

    def new_center(self, pos):
        opos = self.get_true_pos(pos)
        offset = [(self.size[i] / 2) / self.zoom - opos[i] for i in range(2)]
        self.offset = offset

    def get_hovered_place(self, pos):
        tpos = self.get_true_pos(pos)
        if self.current_hovered_place is None:
            for p in self.places:
                check = p.check_hovered(tpos)
                if check:
                    self.current_hovered_place = p
                    break
        else:
            c = self.current_hovered_place
            if not c.check_hovered(tpos):
                self.current_hovered_place = None

    def begin_screen_moving(self, pos):
        self.moving = True
        self.moving_pos = pos
        self.moving_offset = self.offset[:]

    def continue_screen_moving(self, pos):
        self.offset = \
            [int(self.moving_offset[i] +
                 (pos[i] - self.moving_pos[i])
                 / self.zoom) for i in range(2)]

    def end_screen_moving(self):
        self.moving = False

    def begin_place_moving(self, pos):
        self.moving_a_place = True
        tpos = self.get_true_pos(pos)
        self.moving_a_place_pos = pos
        self.temp_obj = self.current_hovered_place
        self.moving_a_place_offset = \
            [tpos[i] - tpos[i] +
             self.temp_obj.pos[i] for
             i in range(2)]
        self.places.remove(self.current_hovered_place)
        self.places.insert(-1, self.current_hovered_place)

    def continue_place_moving(self, pos):
        self.temp_obj.pos = \
            [self.moving_a_place_offset[i] +
             (pos[i] - self.moving_a_place_pos[i])
             / self.zoom for i in range(2)]

    def end_place_moving(self):
        self.moving_a_place = False

    def begin_creating_rect(self):
        self.creating_rect = 1
        self.bgsurface.fill((0, 0, 255))

    def continue_creating_rect(self, pos):
        if self.creating_rect == 1:
            rpos = [self.get_true_pos(pos)[i] -
                    self.current_hovered_place.pos[i]
                    for i in range(2)]
            self.temp_rect = rpos
            self.creating_rect = 2
        elif self.creating_rect == 2:
            arpos = self.temp_rect
            brpos = [self.get_true_pos(pos)[i] -
                     self.current_hovered_place.pos[i]
                     for i in range(2)]
            for i in range(2):
                if arpos[i] > brpos[i]:
                    t = brpos[i]
                    brpos[i] = arpos[i]
                    arpos[i] = t
            size = [brpos[i] - arpos[i] for i in range(2)]
            rect = self.current_hovered_place.create_rect(arpos, size)

            self.creating_rect = False
            self.bgsurface.fill(self.bgcolor)
            self.begin_naming_rect(rect, True)

    def begin_adjusting_rect(self, mod=0):
        self.adjusting_rect_mod = mod
        self.adjusting_rect = 1
        self.bgsurface.fill((0, 222, 222))

    def continue_adjusting_rect(self, pos):
        if self.adjusting_rect == 1:
            self.temp_obj = self.current_hovered_place.highlighted_obj
            if self.temp_obj is not None and self.temp_obj.var == 'rect':
                self.adjusting_rect = 2
        elif self.adjusting_rect == 2:
            rpos = [self.get_true_pos(pos)[i] -
                    self.current_hovered_place.pos[i]
                    for i in range(2)]
            to = self.temp_obj
            if self.adjusting_rect_mod == 1:
                npos = rpos
                nsize = [to.size[i] + to.rpos[i] - rpos[i] for i in range(2)]
            elif self.adjusting_rect_mod == 2:
                npos = to.rpos[:]
                nsize = [rpos[i] - to.rpos[i] for i in range(2)]
            else:
                self.temp_rpos = rpos
                self.adjusting_rect = 3
                return

            # If 1 or 2:
            self.temp_rpos = npos
            self.temp_size = nsize
            self.adjusting_rect = 4
            self.continue_adjusting_rect(pos)
        elif self.adjusting_rect == 3:
            rpos = [self.get_true_pos(pos)[i] -
                    self.current_hovered_place.pos[i]
                    for i in range(2)]
            self.temp_size = [rpos[i] - self.temp_rpos[i] for i in range(2)]
            self.adjusting_rect = 4
            self.continue_adjusting_rect(pos)
        elif self.adjusting_rect == 4:
            npos = self.temp_rpos
            nsize = self.temp_size
            for i in range(2):
                if nsize[i] < 0:
                    nsize[i] = -nsize[i]
                    npos[i] -= nsize[i]
            self.temp_obj.rpos = npos[:]
            self.temp_obj.size = nsize[:]
            self.temp_obj.update()

            self.adjusting_rect = False
            self.bgsurface.fill(self.bgcolor)

    def begin_creating_point(self):
        self.creating_point = 1
        self.bgsurface.fill((0, 255, 0))

    def continue_creating_point(self, pos):
        if self.creating_point == 1:
            rpos = [self.get_true_pos(pos)[i] -
                    self.current_hovered_place.pos[i]
                    for i in range(2)]
            self.current_hovered_place.create_point(rpos)
            self.creating_point = False
            self.bgsurface.fill(self.bgcolor)

    def create_link(self, a, b):
        self.links.append(Link(self, a, b))

    def begin_creating_link(self):
        self.creating_link = 1
        self.bgsurface.fill((100, 150, 200))

    def continue_creating_link(self):
        if self.creating_link == 1:
            self.temp_obj = self.current_hovered_place.highlighted_obj
            if self.temp_obj is not None and not \
                    (self.temp_obj.var == 'rect' and \
                         len(self.temp_obj.linked_with) > 0):
                self.creating_link = 2
        elif self.creating_link == 2:
            ntemp = self.current_hovered_place.highlighted_obj
            if ntemp != self.temp_obj and \
                    ntemp.var != self.temp_obj.var and not \
                    (ntemp.var == 'rect' and \
                         len(ntemp.linked_with) > 0):
                self.create_link(self.temp_obj, ntemp)
                self.creating_link = False
                self.bgsurface.fill(self.bgcolor)

    def begin_naming_rect(self, obj=None, also_name_target_dir=False):
        self.naming_rect = 1
        self.temp_also_name_dir = also_name_target_dir
        if obj is not None:
            self.temp_obj = obj
            self.reset_naming_rect()
        self.bgsurface.fill((200, 150, 100))

    def rect_namer(self, event):
        if event.key in (K_RETURN, K_KP_ENTER, K_ESCAPE):
            self.naming_rect = 2
            self.continue_naming_rect()
        elif event.key == K_BACKSPACE:
            self.temp_obj.name = self.temp_obj.name[:-1]
            self.temp_obj.update()
        else:
            self.temp_obj.name += event.unicode
            self.temp_obj.update()

    def reset_naming_rect(self):
        self.special_key_func = self.rect_namer

    def continue_naming_rect(self):
        if self.naming_rect == 1:
            self.temp_obj = self.current_hovered_place.highlighted_obj
            if self.temp_obj is not None and self.temp_obj.var == 'rect':
                self.reset_naming_rect()
        elif self.naming_rect == 2:
            self.special_key_func = None
            self.naming_rect = False
            self.temp_naming_obj = None
            self.bgsurface.fill(self.bgcolor)
            if self.temp_also_name_dir:
                self.also_name_dir = False
                self.begin_setting_target_dir(self.temp_obj)

    def begin_setting_target_dir(self, obj=None):
        self.setting_target_dir = 1
        if obj is not None:
            self.temp_obj = obj
            self.reset_setting_target_dir()
        self.bgsurface.fill((200, 150, 200))

    def dir_namer(self, event):
        if event.key in (K_RETURN, K_KP_ENTER, K_ESCAPE):
            self.setting_target_dir = 2
            self.continue_setting_target_dir()
        elif event.key == K_BACKSPACE:
            self.temp_obj.targetdir = self.temp_obj.targetdir[:-1]
            self.temp_obj.update()
        else:
            self.temp_obj.targetdir += event.unicode
            self.temp_obj.update()

    def reset_setting_target_dir(self):
        self.temp_obj.targetdir = ''
        self.special_key_func = self.dir_namer

    def continue_setting_target_dir(self):
        if self.setting_target_dir == 1:
            self.temp_obj = self.current_hovered_place.highlighted_obj
            if self.temp_obj is not None and self.temp_obj.var == 'rect':
                self.reset_setting_target_dir()
        elif self.setting_target_dir == 2:
            self.special_key_func = None
            self.setting_target_dir = False
            self.bgsurface.fill(self.bgcolor)

    def begin_deleting_obj(self):
        self.deleting_obj = 1
        self.bgsurface.fill((255, 0, 0))

    def continue_deleting_obj(self):
        if self.deleting_obj == 1:
            try:
                self.current_hovered_place.delete_highlighted_object()
            except Exception:
                self.current_hovered_place.delete_self()
            self.deleting_obj = False
            self.bgsurface.fill(self.bgcolor)

    def begin_deleting_link(self):
        self.deleting_link = 1
        self.bgsurface.fill((255, 0, 132))

    def continue_deleting_link(self):
        if self.deleting_link == 1:
            ls = self.current_hovered_place.highlighted_obj.linked_with
            for l in ls:
                self.links.remove(l)
                for w in l.objs:
                    try:
                        w.linked_with.remove(l)
                    except ValueError:
                        pass

            self.deleting_link = False
            self.bgsurface.fill(self.bgcolor)

    def begin_obj_moving(self):
        self.moving_an_object = 1
        self.bgsurface.fill((55, 55, 132))

    def continue_obj_moving(self, pos):
        if self.moving_an_object == 1:
            self.temp_object = \
                self.current_hovered_place.highlighted_obj
            if self.temp_object is not None:
                self.temp_rpos = self.temp_object.rpos
                self.moving_an_obj_pos = pos
                self.moving_an_object = 2
        elif self.moving_an_object == 2:
            npos = [self.temp_rpos[i] +
                    (pos[i] - self.moving_an_obj_pos[i])
                    / self.zoom for i in range(2)]
            to = self.temp_object
            if npos[0] < 0:
                npos[0] = 0
            if npos[1] < 0:
                npos[1] = 0
            if npos[0] + to.size[0] > to.place.size[0]:
                npos[0] = to.place.size[0] - to.size[0]
            if npos[1] + to.size[1] > to.place.size[1]:
                npos[1] = to.place.size[1] - to.size[1]

            if self.temp_object.var == 'point':
                o = self.temp_object
                o.rrpos = [o.rrpos[i] + npos[i] - o.rpos[i] for i in range(2)]
            self.temp_object.rpos = npos

    def end_obj_moving(self):
        self.moving_an_object = False
        self.bgsurface.fill(self.bgcolor)


    def run(self):
        clock = pygame.time.Clock()
        done = False
        while not done:
            clock.tick(30)

            pos = pygame.mouse.get_pos()
            self.get_hovered_place(pos)
            for event in pygame.event.get():
                if event.type == MOUSEBUTTONDOWN:
                    if event.button == 1:
                        cont = True
                        if self.current_hovered_place is not None:
                            cont = False
                            if self.creating_rect:
                                self.continue_creating_rect(pos)
                            elif self.adjusting_rect:
                                self.continue_adjusting_rect(pos)
                            elif self.creating_point:
                                self.continue_creating_point(pos)
                            elif self.creating_link:
                                self.continue_creating_link()
                            elif self.naming_rect:
                                self.continue_naming_rect()
                            elif self.setting_target_dir:
                                self.continue_setting_target_dir()
                            elif self.deleting_obj:
                                self.continue_deleting_obj()
                            elif self.deleting_link:
                                self.continue_deleting_link()
                            elif self.moving_an_object == 1:
                                self.continue_obj_moving(pos)
                            else: cont = True
                        if cont:
                            if not self.moving:
                                self.begin_screen_moving(pos)

                    elif event.button == 3:
                        if not self.moving_a_place and \
                                self.current_hovered_place is not None:
                            self.begin_place_moving(pos)

                    elif event.button == 4:
                        self.relative_zoom(pos, True)
                    elif event.button == 5:
                        self.relative_zoom(pos, False)

                    elif event.button == 2:
                        self.new_center(pos)

                elif event.type == MOUSEBUTTONUP:
                    if event.button == 1:
                        if self.moving_an_object:
                            self.end_obj_moving()
                        elif self.moving:
                            self.end_screen_moving()
                    elif event.button == 3:
                        if self.moving_a_place:
                            self.end_place_moving()

                elif event.type == MOUSEMOTION:
                    if self.moving_an_object == 2:
                        self.continue_obj_moving(pos)

                    if self.moving:
                        self.continue_screen_moving(pos)

                    if self.moving_a_place:
                        self.continue_place_moving(pos)

                elif event.type == KEYDOWN:
                    if self.special_key_func is not None:
                        self.special_key_func(event)
                        continue

                    nothing = \
                        not self.creating_rect and \
                        not self.adjusting_rect and \
                        not self.creating_point  and \
                        not self.creating_link    and \
                        not self.naming_rect       and \
                        not self.setting_target_dir and \
                        not self.deleting_obj        and \
                        not self.deleting_link        and \
                        not self.moving_an_object

                    if event.key == K_r and nothing:
                        self.begin_creating_rect()
                    if event.key == K_a and nothing:
                        mods = pygame.key.get_mods()
                        if mods & KMOD_SHIFT:
                            mod = 1
                        elif mods & KMOD_CTRL:
                            mod = 2
                        else:
                            mod = 0
                        self.begin_adjusting_rect(mod)
                    elif event.key == K_p and nothing:
                        self.begin_creating_point()
                    elif event.key == K_l and nothing:
                        self.begin_creating_link()
                    elif event.key == K_n and nothing:
                        self.begin_naming_rect()
                    elif event.key == K_d and nothing:
                        self.begin_setting_target_dir()
                    elif event.key == K_DELETE and nothing:
                        self.begin_deleting_obj()
                    elif event.key == K_e and nothing:
                        self.begin_deleting_link()
                    elif event.key == K_m and nothing:
                        self.begin_obj_moving()
                    elif event.key == K_PAGEDOWN:
                        self.relative_zoom(pos, False)
                    elif event.key == K_PAGEUP:
                        self.relative_zoom(pos, True)
                    elif event.key == K_s:
                        mods = pygame.key.get_mods()
                        if mods & KMOD_CTRL:
                            self.save()
                        elif not nothing:
                            self.creating_rect = False
                            self.adjusting_rect = False
                            self.creating_point = False
                            self.creating_link = False
                            self.naming_rect = False
                            self.setting_target_dir = False
                            self.deleting_obj = False
                            self.deleting_link = False
                            self.moving_an_object = False
                            self.bgsurface.fill(self.bgcolor)
                    elif event.key == K_ESCAPE:
                        done = True
                elif event.type == QUIT:
                    done = True

            self.screen.blit(self.bgsurface, (0, 0))
            for plc in self.places:
                plc.draw()
            for l in self.links:
                l.draw()

            pygame.display.flip()
